Package: rcatfish
Version: 1.0.2
Date: 2026-2-18
Title: An R Interface to the California Academy of Sciences Eschmeyer's
        Catalog of Fishes
Authors@R: 
  c(person(given = "Samuel R.",
           family = "Borstein",
           role = c("aut", "cre"),
           email = "sam@borstein.com"),
    person(given = "Brandon",
           family = "Dominy",
           role = "aut",
		   email = "bed47@txstate.edu"),
    person(given = "Brian",
           family = "O'Meara",
           role = "aut",
		   email = "bomeara@utk.edu"))
Depends: R (>= 4.4.0)
Imports: curl (>= 5.2.1), httr (>= 1.4.7), jsonlite (>= 1.8.9),
        lubridate (>= 1.9.3), magrittr (>= 2.0.3), qdapRegex (>=
        0.7.2), RCurl (>= 1.95-4.11), rfishbase (>= 5.0.0), rvest (>=
        0.3.2), stringr (>= 1.3.1), tidyr (>= 0.8.1), xml2 (>= 1.2.0)
Description: Accesses the California Academy of Sciences Eschmeyer's Catalog of Fishes in R using web requests. The Catalog of fishes is the leading authority in fish taxonomy. Functions in the package allow users to search for fish taxa and valid names, retrieve taxonomic references, retrieve monthly taxonomic changes, obtain natural history collection information, and see the number of species by taxonomic group. For more information on the Catalog: Fricke, R., Eschmeyer, W. N. & R. van der Laan (eds) 2025. ESCHMEYER'S CATALOG OF FISHES <https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp>.
License: GPL (>= 2)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-02-18 15:25:21 UTC; Sam
Author: Samuel R. Borstein [aut, cre],
  Brandon Dominy [aut],
  Brian O'Meara [aut]
Maintainer: Samuel R. Borstein <sam@borstein.com>
Repository: CRAN
Date/Publication: 2026-02-19 09:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:50:10 UTC; unix

# Generated by roxygen2: do not edit by hand

export(rcatfish_classification)
export(rcatfish_collections)
export(rcatfish_glossary)
export(rcatfish_journals)
export(rcatfish_references)
export(rcatfish_search)
export(rcatfish_species_by)
export(rcatfish_updates)
export(rcatfish_version)
import(rvest)
import(stringr)
importFrom(magrittr,"%>%")
importFrom(xml2,read_html)

######################################################################################################################################
######################################################################################################################################
#
# Helper Functions
#
######################################################################################################################################


######################################################################################################################################
## Parse Distribution Data
######################################################################################################################################
get_distribution <- function(local.result) {
  tryCatch(
    expr = {result <- gsub("U\\.S\\.A\\.", "UnitedStatesOfAmerica", local.result) #Replace U.S.A. with placeholder, temporarily
    result <- gsub("U\\.K\\.", "UnitedKingdom", result) #Replace U.S.A. with placeholder, temporarily
    result <- stringr::str_extract(result, "Distribution: .*? Habitat:")#find distribution
    result <- gsub("Distribution: ", "", result)#split distribution from data
    result <- gsub("Habitat:", "", result) #remove habitat
    result <- str_remove(result, "[Ii][Uu][Cc][Nn].*?:.*")
    result <- gsub("\\.$", "", result)#remove period
    result <- gsub("Habitat:.*|<\\/p>", "", result) #Clean up end of results
    result <- gsub("UnitedStatesOfAmerica", "U\\.S\\.A\\.", result) #Undo USA placeholder
    result <- gsub("UnitedKingdom", "U\\.K\\.", result) #Undo UK placeholder
    if (grepl("<", result, fixed = T)) {result <- StripFun(result)}
    result <- trimws(result)
    return(result)},
    
    error = function(cond) {return("Error parsing distribution")})
}

######################################################################################################################################
## Parse Habitat Data
######################################################################################################################################
get_habitat <- function(local.result) {
  tryCatch(expr = {
    result <- stringr::str_extract(local.result, "Habitat: .*?\\.")#Find habitat flag
    result <- gsub("Habitat: ", "", result)#Remove habitat flag
    result <- gsub("\\.", "", result)#remove period
    HabDat <- as.numeric(sapply(c("freshwater","brackish","marine"), grepl, result))#Get habitats
    HabDat <- ifelse(HabDat==c(0,0,0), yes = rep(NA,3), no =  HabDat)
    return(HabDat)},#return data
    
    error = function(cond) {return("Error parsing habitat")})
}

######################################################################################################################################
## Parse Family/Subfamily Data
######################################################################################################################################
get_family <- function(local.result) {
  tryCatch(expr = {
    result <- gsub("Distribution: .*","",local.result)#remove record from distribution flag on
    result <- gsub("Habitat: .*","",result)#remove record from habitat flag on
    result <- trimws(gsub("<\\/p>|<\\/b>", "",result,fixed = FALSE))#remove last flag if no distribution or habitat data is present
    family.dat <- rev(strsplit(result, "\\.")[[1]])[1]#split on period and take the family info
    result_strings <- sapply(family.dat, strsplit, ": ")#split family/subfamily
    result_strings <- sapply(result_strings, trimws) #trim whitespace
    result_strings[1] <- ifelse(grepl("idae$", result_strings[1]), result_strings[1], NA) #confirm family has suffix -idae
    result_strings[2] <- ifelse(grepl("inae$", result_strings[2]), result_strings[2], NA) #confirm subfamily has suffix -inae
    result_strings[1] <- ifelse(grepl(" ", result_strings[1]),  utils::tail(unlist(str_split(result_strings[1], " ")), 1), result_strings[1]) #extract family if extra data has been included
    result_strings[2] <- ifelse(grepl(" ", result_strings[2]),  utils::tail(unlist(str_split(result_strings[2], " ")), 1), result_strings[2]) #extract family if extra data has been included
    return(unname(unlist(result_strings)))},#return unlisted unnamed family info
    
    error = function(cond) {return(c("Error parsing family", "Error parsing subfamily"))})
}

######################################################################################################################################
## Parse Status Data
######################################################################################################################################
get_current_status <- function(local.result) {
  tryCatch(expr = {
    result <- stringr::str_extract(local.result, "Current status.*") #Find status flag
    result <-  stringr::str_split(string = result,pattern = "\\.")[[1]] #split at periods
    if (grepl("^ \\&|^ \\d{4}|^, ", result[2])) {result <- paste0(result[1], fixed("."), result[2])} #Fix double authors
    else if (grepl("et al$", result[1])) {result <- paste0(result[1], fixed("."), result[2])} #Fix et al.
    else {result <- result[1]}
    if (grepl("</b> Uncertain$", result)) {
      status <- "Uncertain"
      species <- NA
    } else if (grepl("</b> Unknown$", result)) {
      status <- "Unknown"
      species <- NA
    } else if (grepl("</b> Synonym$", result)) {
      status <- "Synonym"
      species <- NA 
    } else {
      status<-trimws(gsub("</b>|<i>","", stringr::str_extract(result, "</b>.+<i>")))#get the status
      species<-gsub("<i>|</i>","", stringr::str_extract(result, "<i>.+</i>"))#get the recognied species name
    }
    ifelse(is.na(species) | is.na(stringr::str_extract(result, "</i>.+")),author <- NA, author <- trimws(StripFun(stringr::str_extract(result, "</i>.+"))))
    return(c(status,species,author))},
    
    error = function(cond) {return(c("Error parsing status", "Error parsing species", "Error parsing author"))})
}

######################################################################################################################################
## Get Focal Taxon Name and Author of Description
######################################################################################################################################
get_focal_name <- function(local.result){
  tryCatch(expr = {
    focal.binom <- StripFun(stringr::str_extract(local.result, "<b><i>.+</i></b>"))#Get nominal name
    focal.binom <- strsplit(focal.binom,",")#split nominal name elements
    binom.maker<-function(x){#funtion to make binomial
      paste(rev(x), collapse = " ")
    }
    focal.binom.final <- trimws(lapply(focal.binom, binom.maker))#make binomial
    if (grepl("[12]\\d\\d\\d\\-\\d\\d:", local.result)) {
      Author = qdapRegex::ex_between(local.result, left = "</i></b> ",right = "[12]\\d\\d\\d\\-\\d\\d",fixed = FALSE,include.markers = TRUE)
      DescriptionYear = str_extract(string = Author, pattern = "[12]\\d\\d\\d\\-\\d\\d")
    } else {
      Author <- qdapRegex::ex_between(local.result, left = "</i></b> ",right = "[12]\\d\\d\\d",fixed = FALSE,include.markers = TRUE)#extract author year
      DescriptionYear <-  stringr::str_extract(string = Author, pattern = "[12]\\d\\d\\d")
    }
    Author <- trimws(gsub("amp;|</i>|</b>","",Author))#remove xml flags from author name
    Description <- qdapRegex::ex_between(local.result, left = " ref.",right = "]")[[1]][1]#get description source
    DescriptionRef <- qdapRegex::ex_between(Description,left = "\">",right = "</a>")#get reference number
    return(unlist(c(focal.binom.final,Author, DescriptionRef, DescriptionYear)))},
    
    error = function(cond) {return(c("Error parsing focal name", "Error parsing focal author", "Error parsing description reference", "Error parsing description year"))})
}

######################################################################################################################################
## Parse Type Information
######################################################################################################################################
get_types <- function(local.result){
  tryCatch(expr = {
    type.vec<-rep(NA,7)#make vector to store type information
    names(type.vec)<-c("Holotype","Paratype","Lectotype","Paralectotype","Neotype","Syntype", "NoTypes")#name types
    
    replacements <- c(" Ea\\. ", " Uncat\\. ", " ea\\. ", " uncat\\. ", " Fish\\. ",
                      " No\\. ", " Orig\\. ", " no\\. ", " orig\\. ", " Cat\\. ",
                      " P\\. ", " p\\. ", " coll\\. ", " cat\\. ", " Scot\\. ",
                      "\\(Orig\\. ", " Coll\\. ", "\\(orig\\. ", " Edinb\\. ",
                      " Univ\\. ", " univ\\. ", " mus\\. ", " Mus\\. ", "-zool\\. ",
                      " Pl\\. ", "\\(Ca\\. ", " pl\\. ", "\\(ca\\. ", "\\[orig\\. ",
                      "\\[Ex\\. ", " Inst\\. ", "\\[ex\\. ", " inst\\. ",
                      " Hydrobiol\\. ", " Zool\\. ", " hydrobiol\\. ", " zool\\. ",
                      " Dept\\. ", " Agric\\. ", " dept\\. ", " agric\\. ",
                      " Acad\\. ", " Sci\\. ", " Ist\\. ", " acad\\. ",
                      " sci\\. ", " ist\\. ", "\\>m\\. ", "\\>M\\. ", " Ex\\. ", " ex\\. ",
                      "\\[Mus\\. ", " Biol\\. ", " Lab\\. ", " Soc\\. ", " Sci\\. ", " Imp\\. ",
                      "65\\. \\(1, sk", "3-64\\. \\(2\\)", " ca\\. ", "898\\.9\\.9\\. 40", 
                      "90 \\(1\\), 2318 \\(1\\)\\. 53", "ens\\)\\. CF", " spec\\. ", " E\\.A\\.F\\.R\\.O\\. ",
                      "\\(ex\\. ", "MNHN 2011-0232\\. M", "7\\. now 6", "57 \\(2\\)\\. P023",
                      "so paratype of P\\. ri", "pes of L\\. wern", " Depto\\. ", " depto\\. ", "RAC\\. ZSM\\.",
                      " Geol\\. ", " Nat\\. ", " Hist\\. ", " Surv\\. ", "Belfast, U\\.K\\. \\(1",
                      "348 \\(1\\)\\. 1029", " U\\.S\\. ", " Bur\\. ", " Fish\\. ", " Pref\\. ",
                      " Exper\\. ", " Mem\\. ", " CAS\\]\\. CAS", " Prov\\. ", "IA1 H\\. N", "82\\-04 \\(1\\)\\. H",
                      " Res\\. ", "\\[Stadt\\. ", " MCZ\\)\\. ")
    for (i in seq_along(replacements)) {
      newString <- str_replace(replacements[i], "\\. ", "xxyyzz ")
      local.result <- str_replace_all(local.result, replacements[i], newString)
    }
    
    HolotypeCheck <- grep("Holotype",local.result)#check for holotypes and extract if present
    if(length(HolotypeCheck)>0){
      type.vec[1] <- qdapRegex::ex_between(local.result[HolotypeCheck], left = c("Holotype \\(unique\\):","Holotype:","Holotype \\(\\?\\):", "Holotype \\(unique\\) \\(\\?\\):"), right = c("\\. ","\\. ", "\\. ", "\\. "), fixed = FALSE)
    }
    ParatypeCheck <- grep("Paratype", local.result)#check for paratypes and extract if present
    if(length(ParatypeCheck)>0){
      type.vec[2] <- qdapRegex::ex_between(local.result[ParatypeCheck], left = c("Paratype:","Paratypes:", "Paratype \\(\\?\\):", "Paratypes \\(\\?\\):", "Paratypes \\(listed as additional material\\):", "Paratypes \\(11\\):"), right = c("\\. ","\\. ","\\. ","\\. ", "\\. ", "\\. "), fixed = FALSE)
      type.vec[2] <- str_remove(type.vec[2], "First appeared in Nguyen et al")
    }
    LectotypeCheck <- grep("Lectotype",local.result)#check for lectotypes and extract if present
    if(length(LectotypeCheck)>0){
      type.vec[3] <- qdapRegex::ex_between(local.result[LectotypeCheck], left = c("Lectotype:","Lectotypes:"), right = c("\\. ","\\. "), fixed = FALSE)
    }
    ParalectotypeCheck <- grep("Paralectotype",local.result)#check for paralectotypes and extract if present
    if(length(ParalectotypeCheck)>0){
      type.vec[4] <- qdapRegex::ex_between(local.result[ParalectotypeCheck], left = c("Paralectotype:","Paralectotypes:"), right = c("\\. ","\\. "), fixed = FALSE)
    }
    NeotypeCheck <- grep("Neotype",local.result)#check for neotypes and extract if present
    if(length(NeotypeCheck)>0){
      type.vec[5] <- qdapRegex::ex_between(local.result[NeotypeCheck], left = c("Neotype:","Neotypes:"), right = c("\\. ","\\. "), fixed = FALSE)
    }
    SyntypeCheck <- grep("Syntype",local.result, ignore.case = TRUE)#check for syntypes and extract if present
    if(length(SyntypeCheck)>0){
      type.vec[6] <- qdapRegex::ex_between(local.result[SyntypeCheck], left = c("Syntype:","Syntypes:", "Syntypes \\(\\?\\):", "Syntypes \\(\\?\\) \\[or one is the holotype\\]:", "syntypes:"), right = c("\\. ","\\. ","\\. ","\\. ","\\. "), fixed = FALSE)
    }
    NoTypesCheck <- grep("No types known|No type known",local.result) #Check for no types known and extract if present
    if(length(NoTypesCheck)>0){
      if(grepl("No types known",local.result)) {
        type.vec[7] <- "No types known"
      }
      if(grepl("No type known",local.result)) {
        type.vec[7] <- "No type known"
      }
    }
    type.vec <- str_replace_all(type.vec, "xxyyzz", "\\.")
    local.result <- str_replace_all(local.result, "xxyyzz", "\\.")
    type.vec <- stringr::str_replace_all(type.vec, fixed("&amp;"), fixed("&"))
    type.vec <- stringr::str_remove_all(type.vec, "\\u2022.*")
    type.vec <- stringr::str_remove(type.vec, "Type catalog:.*")
    type.vec <- stringr::str_remove(type.vec, "Type cataqlog:.*")
    type.vec <- stringr::str_remove_all(type.vec, "\\<i\\>|\\<\\/i\\>")
    return(type.vec)},
    
    error = function(cond) {return(rep("Error parsing types", times = 7))})
}

######################################################################################################################################
## Get Type Locality
######################################################################################################################################
get_type_locality <- function(local.result){
  tryCatch(expr = {
    locality <- qdapRegex::ex_between(local.result, left = c("</a>]","</a>]"), right = c(" *type"," *types "), fixed = FALSE)[[1]][1]#Get type locality
    locality <- gsub(" [Hh]olo$| [Ss]yn$| [Ll]ecto$| [Pp]ara$| [Pp]aralecto$| [Nn]eo$| No$| Valenciennes$", "",locality)#strip out trailing info on types
    if (grepl("^Holo$|^Syn$|^Lecto$|^Paralecto$|^Para$|^Neo$|^No$", locality)) {locality <- NA}
    locality <- trimws(locality)
    locality <- str_remove(locality, " Plus non\\-") #remove plus 'non-type material' in mistypes
    locality <- str_remove(locality, " Possible$")
    locality <- str_remove(locality, "Species illustrated and described in more detail.*?$")
    locality <- str_remove(locality, "Not translated, perhaps")
    locality <- stringr::str_replace_all(locality, fixed("&amp;"), fixed("&"))
    locality <- str_remove(locality, "Non-$")
    if (is.na(locality)) {
      locality <- qdapRegex::ex_between(local.result, left = "</a>]", right = "\\u2022", fixed = FALSE)[[1]][1]#Get type locality
      locality <- unlist(strsplit(locality, "\\. "))
      locVec <- c()
      for (i in 1:nrow(GeographicTerms)) {
        locFind <- grep(str_remove_all(GeographicTerms[i,1], "\\(|\\)"), locality)
        if (length(locFind) > 0) {
          locVec <- c(locVec, locFind)
        }
      }
      locVec <- locVec[1]
      locality <- locality[locVec]
    }
    locality <- str_remove(locality, "\\.$")
    if (length(locality) == 0) {locality <- ""}
    if (grepl("<", locality, fixed = T)) {locality <- StripFun(locality)}
    if (locality == "") {locality <- NA} 
    return(locality)},
    
    error = function(cond) {return("Error parsing locality")})
}

######################################################################################################################################
## Get Gender
######################################################################################################################################
get_gender <- function(local.result){
  tryCatch(expr = {
    genders2mine <- c("Masc.","Fem.","Neut.")#Specify genders to mine
    gender <- NA#if no gender, set as NA and if gender is found set it
    #loop to find gender
    for(gender.index in 1:3) {
      ifelse(length(grep(genders2mine[gender.index],local.result))==0, gender <- NA, gender <- genders2mine[gender.index])
      if (!is.na(gender)) {
        break
      }
      ifelse(length(grep(genders2mine[gender.index],local.result))==0, gender <- NA, gender <- genders2mine[gender.index])
      if (!is.na(gender)) {
        break
      }
      ifelse(length(grep(genders2mine[gender.index],local.result))==0, gender <- NA, gender <- genders2mine[gender.index])
    }
    return(gender)},
    
    error = function(cond) {return("Error parsing gender")})
}

######################################################################################################################################
## Get Focal Genus
######################################################################################################################################
get_focal_genus <- function(local.result){
  tryCatch(
    expr = {  genus <- qdapRegex::ex_between(local.result, left = "<b><i>","</i></b>")[[1]]#Get focal genus names
    Author <- qdapRegex::ex_between(local.result, left = "</i></b> ",right = ":")#Get author of description
    genus.note <- stringr::str_replace_all(qdapRegex::ex_between(local.result, left = "<\\/b> (",right = ")")[[1]][1],"<i>|</i>|<b>|</b>","")#check for notes (i.e., subgenus)
    Author <- trimws(gsub("\\s*\\([^\\)]+\\)", "",Author,fixed = FALSE))#remove author and trim spaces
    Author <- gsub("amp;","",Author)#remove xml & flags
    DescriptionYear <- stringr::str_extract(string = Author, pattern = "[12]\\d\\d\\d")
    Description<- qdapRegex::ex_between(local.result, left = "; ref.",right = "]")#remove reference for description
    DescriptionRef <- qdapRegex::ex_between(Description,left = "\">",right = "</a>")#obtain reference numbers
    return(unlist(c(genus,Author,DescriptionRef, DescriptionYear, genus.note)))},
    
    error = function(cond) {return("Error parsing focal genus")})
}

######################################################################################################################################
## Get Type Species
######################################################################################################################################
get_type_species <- function(local.result){
  tryCatch(
    expr = {type.species <- qdapRegex::ex_between(local.result, left = ". <i>","</i>")#get the species using italics flgs
    return(type.species)},
    
    error = function(cond) {return("Error parsing type species")})
}

######################################################################################################################################
## Get Type by
######################################################################################################################################
get_type_by <- function(local.result){
  tryCatch(
    expr = {  type.species <- qdapRegex::ex_between(local.result, left = "Type by",".",include.markers = TRUE)#Find type by flag and remove
    return(type.species)},
    
    error = function(cond) {return("Error parsing type by")})
}

######################################################################################################################################
## Get Genus Family
######################################################################################################################################
get_family_genus <- function(local.result) {
  tryCatch(
    expr = {  mine.family <- qdapRegex::ex_between(local.result, left = "<b>Current status:</b> ","</p>",include.markers = FALSE)#Find current status info
    family <- qdapRegex::ex_between(mine.family, left = ". ",".",include.markers = FALSE)#separate out family info
    result_strings <- sapply(family, strsplit, ": ")#separate family/subfamily
    return(unlist(result_strings))},
    
    error = function(cond) {return("Error parsing family genus")})
}

######################################################################################################################################
## Get Genus Notes
######################################################################################################################################
get_genus_notes <- function(local.result) {
  tryCatch(expr = {
    find.note<-qdapRegex::ex_between(local.result, left = "Type ","\\u2022",include.markers = TRUE)#get note section
    notes <- gsub("Type by.*?\\.\\s\\s","",find.note)#remove type by info
    for (note.index in seq_along(notes)) {
      if(is.na(notes[note.index])|nchar(notes[note.index])<=1){
        notes[note.index] <- NA#if no notes, set to NA
      }else{
        if (grepl("<.*?>", notes[note.index])) {
          notes[note.index] <- trimws(gsub(pattern = "\u2022", replacement = "", StripFun(notes[note.index]),fixed = TRUE))#if notes are present in html form, extract and trim
        } else {
          notes[note.index] <- trimws(gsub(pattern = "\u2022", replacement = "", notes[note.index],fixed = TRUE))#if notes are present otherwise, extract and trim
        }
      }
    }
    return(notes)},
    error = function(cond) {return("Error parsing genus notes")})
}

######################################################################################################################################
## Get Nomenclature Notes
######################################################################################################################################
get_nomenclature_notes <- function(local.result) {
  tryCatch(expr = {
    result <- gsub(pattern = "<b><i>.*<\\/i><\\/b>", replacement = "", local.result) # Remove genus and species
    result <- stringr::str_extract(result, "<b>.*<\\/b>") # Get all bolded text
    result <- trimws(gsub("<b>|<\\/b>|Current status|:|\\.", "", result))
    for (i in seq_along(result)) {
      if (identical("",result[i])) {
        result[i] <- NA
      } else {
        next
      }
    }
    return(result)},
    error = function(cond) {return("Error parsing nomenclature notes")})
}

######################################################################################################################################
## Get IUCN Notes
######################################################################################################################################
get_iucn <- function(local.result) {
  tryCatch(
    expr = {
      result <- stringr::str_extract(local.result, "[Ii][Uu][Cc][Nn].*?:.*?\\.")
      if (grepl(".*?:.*?\\(", result)) {
        result <- stringr::str_extract(local.result, "[Ii][Uu][Cc][Nn].*?:.*?\\(.*?\\).*?\\.")
      }
      result <- trimws(gsub(pattern = "Habitat:.*", replacement = "", x = result))
      result <- gsub("[Ii][Uu][Cc][Nn]|\\.|<i>|<\\/i>", "", result)
      year <- stringr::str_extract(result, "\\d{4}")
      result <- str_remove(result, "^.*?:")
      result <- str_remove(result, ",$")
      status <- trimws(gsub("\\d{4}", "", result))
      status <- trimws(str_remove(status, "^\\(\\)[; ]"))
      return(matrix(data = c(year, status), nrow = length(year), ncol = 2))},
    
    error = function(cond) {return(c("Error parsing IUCN year", "Error parsing IUCN status"))})
}

######################################################################################################################################
## Resolve Names
######################################################################################################################################
Resolve_Names <- function(NameInput){
  #Below pings the Global Names Verifier API to perform a query
  VerifyNames <- httr::POST("https://verifier.globalnames.org/api/v1/verifications", 
                            config = httr::add_headers(accept = "application/json", `Content-Type` = "application/json"),
                            body = jsonlite::toJSON(list(nameStrings=c(NameInput))))
  #Convert to machine readable
  VerifyNames.Res <- jsonlite::fromJSON(rawToChar(VerifyNames$content))
  if(!VerifyNames.Res$names$matchType=="NoMatch"){#if result is anything other than no match, proceed
    if(VerifyNames.Res$names$cardinality == VerifyNames.Res$names$bestResult$matchedCardinality){#Check cardinality match
      return(VerifyNames.Res$names$bestResult$matchedCanonicalFull)
    }else{
      message(paste0("Unable to resolve query ", NameInput))
    }
  }else{
    message(paste0("Unable to resolve query ", NameInput))
  }
}

######################################################################################################################################
## Get Infrasubspecific Tags
######################################################################################################################################
get_infrasub <- function(local.author) {
  tryCatch(
    expr = {
      infrasub_tags <- c("^natio ", "^var\\. ", "^forma ", "^form ", "^morpha ", "^infra. ", "^subform ", "^race ", "^type ", "^gruppe ", "^aberr. ", "^facies ") #Vector of potential tags to find
      
      new.author <- local.author
      infrasub <- NA
      
      for (infra_index in seq_along(infrasub_tags)) {
        if (grepl(pattern = infrasub_tags[infra_index], x = local.author)) {
          new.author <- str_remove(local.author, infrasub_tags[infra_index])
          infrasub <- trimws(str_extract(local.author, infrasub_tags[infra_index]))
        } else {
          next
        }
      }
      
      return(unlist(c(new.author, infrasub)))},
    
    error = function(cond) {return(c(local.author, "Error parsing infrasubspecific tags"))})
}

######################################################################################################################################
## Get Miscellaneous Data
######################################################################################################################################
get_miscellanea <- function(raw.input, processed.input) {
  tryCatch(
    expr = {
      taxon.history_data <- unlist(qdapRegex::ex_between(raw.input,c("\\u2022","\\u2022"),c(". <b>"," <b>"))) #Pull data used for taxon history
      if (!is.na(taxon.history_data) && grepl(fixed("<"), taxon.history_data)) {
        taxon.history_data <- rvest::html_text(xml2::read_html(taxon.history_data)) #Convert taxon.history info to text
      } 
      final.output <- rvest::html_text(xml2::read_html(raw.input)) #convert raw data to text
      
      #Remove nominal taxa
      if (!is.na(processed.input$NominalTaxa)) {
        name_components <- unlist(str_split(processed.input$NominalTaxa, " "))
        binomial_min <- utils::tail(name_components, 1)
        binomial_rest <- paste(name_components[1:length(name_components)-1], collapse = " ")
        full_binomial <- paste("^", binomial_min, ", ", binomial_rest, sep = "")
        full_binomial <- str_replace(full_binomial, fixed("("), "\\(")
        full_binomial <- str_replace(full_binomial, fixed(")"), "\\)")
        final.output <- trimws(str_remove(final.output, full_binomial))
      }
      #Remove author and year
      if (!is.na(processed.input$Author)) {
        final.output <- str_remove(final.output, fixed(processed.input$Author))
      }
      #Remove habitat data
      if (is.na(processed.input$Fresh) && is.na(processed.input$Brackish) && is.na(processed.input$Marine)) {
      } else if (processed.input$Fresh == 1 || processed.input$Brackish == 1 || processed.input$Marine == 1) {
        final.output <- str_remove(final.output, "Habitat: .*?\\.")
      }
      #Remove infrasubspecific tags
      if (!is.na(processed.input$Infrasubspecific)) {
        infra_tag <- processed.input$Infrasubspecific
        infra_tag <- paste("^", str_replace(infra_tag, fixed("."), "\\."), sep = "")
        final.output <- str_remove(final.output, infra_tag)
      }
      #Remove distribution data
      if (!is.na(processed.input$Distribution)) {
        final.output <- str_remove(final.output, fixed(paste("Distribution:", processed.input$Distribution)))#remove distribution
      }
      #Remove family/subfamily
      if (!is.na(processed.input$Family)) {
        if (!is.na(processed.input$Subfamily)) {
          final.output <- str_remove(final.output, pattern = paste(processed.input$Family, ": ", processed.input$Subfamily, "\\.", sep = ""))
        } else {
          final.output <- str_remove(final.output, pattern = paste(processed.input$Family, "\\.", sep = ""))
        }
      }
      #Remove current status, current authority, and current nomenclature
      if (!is.na(processed.input$Status)) {
        if (processed.input$Status == "Unknown" || processed.input$Status == "Uncertain") {
          currentStatus <- paste(fixed("Current status:"), processed.input$Status)
        }
        else if (!is.na(processed.input$CurrentAuthority)) {
          currentStatus <- paste(fixed("Current status:"), processed.input$Status, processed.input$CurrentNomenclature, processed.input$CurrentAuthority)
        } else {
          currentStatus <- paste(fixed("Current status:"), processed.input$Status, processed.input$CurrentNomenclature)
        }
        final.output <- str_remove_all(final.output, fixed(currentStatus))
      }
      #Remove nomenclature notes
      if (!is.na(processed.input$NomenclatureNotes)) {
        final.output <- str_remove(final.output, paste("\\. ", processed.input$NomenclatureNotes, "\\.", sep = ""))
      }
      #Remove description refernece 
      if (!is.na(processed.input$DescriptionRef)) {
        description <- paste("\\[.*?", processed.input$DescriptionRef, "\\]", sep = "")
        #description <- str_extract(final.output, description)
        final.output <- str_remove(final.output, description)
      }
      #Remove type locality
      if (!is.na(processed.input$TypeLocality)) {
        final.output <- str_remove(final.output, fixed(processed.input$TypeLocality))
      }
      #Remove types
      if (!is.na(processed.input$Holotype)) {
        final.output <- str_remove(final.output, fixed(paste("Holotype:", processed.input$Holotype)))
        final.output <- str_remove(final.output, fixed(paste("Holotype (unique):", processed.input$Holotype)))
        final.output <- str_remove(final.output, fixed(paste("Holotype (?):", processed.input$Holotype)))
        final.output <- str_remove(final.output, fixed(paste("Holotype (unique) (?):", processed.input$Holotype)))
      }
      if (!is.na(processed.input$Paratype)) {
        final.output <- str_remove(final.output, fixed(paste("Paratype:", processed.input$Paratype)))
        final.output <- str_remove(final.output, fixed(paste("Paratypes:", processed.input$Paratype)))
        final.output <- str_remove(final.output, fixed(paste("Paratypes: ", processed.input$Paratype)))
        final.output <- str_remove(final.output, fixed(paste("Paratypes (listed as additional material):", processed.input$Paratype)))
        final.output <- str_remove(final.output, fixed(paste("Paratypes (11):", processed.input$Paratype)))
        
      }
      if (!is.na(processed.input$Lectotype)) {
        final.output <- str_remove(final.output, fixed(paste("Lectotype:", processed.input$Lectotype)))
        final.output <- str_remove(final.output, fixed(paste("Lectotypes:", processed.input$Lectotype)))
      }
      if (!is.na(processed.input$Paralectotype)) {
        final.output <- str_remove(final.output, fixed(paste("Paralectotype:", processed.input$Paralectotype)))
        final.output <- str_remove(final.output, fixed(paste("Paralectotypes:", processed.input$Paralectotype)))
      }
      if (!is.na(processed.input$Neotype)) {
        final.output <- str_remove(final.output, fixed(paste("Neotype:", processed.input$Neotype)))
        final.output <- str_remove(final.output, fixed(paste("Neotypes:", processed.input$Neotype)))
      }
      if (!is.na(processed.input$Syntype)) {
        final.output <- str_remove(final.output, fixed(paste("Syntype:", processed.input$Syntype)))
        final.output <- str_remove(final.output, fixed(paste("Syntypes:", processed.input$Syntype)))
      }
      if (!is.na(processed.input$NoTypes)) {
        final.output <- str_remove(final.output, processed.input$NoTypes)
      }
      #Remove IUCN Year and Status
      if (!is.na(processed.input$IUCNYear) || !is.na(processed.input$IUCNStatus)) {
        if (grepl("\\(", processed.input$IUCNStatus)) {
          final.output <- str_remove(final.output, "[Ii][Uc][Cc][Nn].*?:.*?\\(.*?\\).*?\\.")
        } else {
          final.output <- str_remove(final.output, "[Ii][Uu][Cc][Nn].*?:.*?\\.")
        }
      }
      #Remove items that should theoretically be pulled by taxon history
      final.output <- str_remove(final.output, fixed(taxon.history_data))
      final.output <- str_remove(final.output, "\\u2022")
      #Remove page numbers
      if (!is.na(processed.input$RefPage)) {
        capturedPage <- str_replace_all(processed.input$RefPage, fixed("("), replacement = "\\(")
        capturedPage <- str_replace_all(capturedPage, fixed(")"), replacement = "\\)")
        capturedPage <- str_replace_all(capturedPage, fixed("."), replacement = "\\.")
        capturedPage <- str_replace_all(capturedPage, fixed("["), replacement = "\\[")
        capturedPage <- str_replace_all(capturedPage, fixed("]"), replacement = "\\]")
        capturedPage <- str_replace_all(capturedPage, fixed("?"), replacement = "\\?")
        capturedPage <- str_replace_all(capturedPage, fixed("+"), replacement = "\\+")
        final.output <- str_remove(final.output, paste(":", capturedPage, sep = ""))
      }
      
      #Clean up results
      final.output <- str_replace_all(final.output, "(\\. )+", "\\. ") # Clean up repeated periods
      final.output <- str_remove(final.output, "[\\. ]*$") # Clean excess spaces/periods at end of string
      final.output <- str_remove(final.output, "^[ \\.]*") # Clean excess spaces/periods at start of string
      
      if (!is.na(final.output)) {
        if (nchar(as.character(final.output)) == 0) {final.output <- NA} # Change blanks to NA
      }
      
      return(trimws(final.output))
    },
    
    error = function (cond) {return(paste("Error parsing miscellaneous data", cond))}
  )
}

######################################################################################################################################
## Get Page Number and Figure Data
######################################################################################################################################
get_page_number <- function(local.result, processed.input) {
  pageNum <- qdapRegex::ex_between(StripFun(local.result), left = processed.input, right = "[", fixed = TRUE)
  pageNum <- str_remove(pageNum, "^:")
  return(pageNum)
}

######################################################################################################################################
## Get Scientific Names From Common Names
######################################################################################################################################
GetSci <- function(query = query, language = language){
  message(paste0("Now matching common names to scientific names"))
  SciNames <- data.frame(matrix(nrow = 0, ncol = 3))
  colnames(SciNames) <- c("Query","Species","ComName")
  for(i in 1:length(query)){
    NameQuery <- rfishbase::common_to_sci(query[i], Language = language)
    Current.Dat <- cbind.data.frame(rep(query[i],dim(NameQuery)[1]),NameQuery$Species,NameQuery$ComName)
    SciNames <- rbind.data.frame(SciNames,Current.Dat)
  }
  return(SciNames)
}

######################################################################################################################################
## Search Catalog to Retrieve local.result 
######################################################################################################################################
catalog_search <- function(query, type, unavailable = FALSE, resolve = FALSE, phrase = FALSE) {
  ##### Add Headers for httr POST Requests #####
  headers = c(
    `Content-Type` = "application/x-www-form-urlencoded"
  )
  
  ##### Obtain Data from Cal Acad #####
  FormData = list(
    `name` = "searchform",
    `contains` = ifelse(phrase == TRUE, shQuote(query, type = "cmd"), query),
    `tbl` = type,
    `Submit` = "Search",
    `unavailable` = ifelse(unavailable == F, "Off", "On")
  )
  result <- httr::POST(url = "https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp", httr::add_headers(.headers=headers), body = FormData, encode = "form", config = httr::config(ssl_verifypeer = FALSE))
  
  ##### Parse Result Into Usable Format #####
  result.rv <- xml2::read_html(result) # Parse results into html
  local.result <- as.character(rvest::html_elements(result.rv, '.result')[-1]) # Strip out header lines
  local.result <- local.result[!grepl('<p class=\"result\">(\r)?\n</p>\n', local.result)] # Strip out returns and form into list 
  
  ##### Resolve Names if Necessary #####
  if((length(local.result) == 0) && (resolve == TRUE)){
    resolved.names <- Resolve_Names(NameInput = query)#Try fuzzy searching and resolving names
    if(length(resolved.names) > 0) {
      message(paste0("No results found for supplied taxon ", query, ".", " Trying the best resolved name ", resolved.names, "."))
      FormData = list(
        `name` = "searchform",
        `contains` = ifelse(phrase == TRUE, shQuote(resolved.names, type = "cmd"), resolved.names),
        `tbl` = type,
        `Submit` = "Search",
        `unavailable` = ifelse(unavailable == F, "Off", "On")
      )
      result <- httr::POST(url = "https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp", httr::add_headers(.headers=headers), body = FormData, encode = "form", config = httr::config(ssl_verifypeer = FALSE))
      result.rv <- xml2::read_html(result)#parse results
      local.result <- as.character(rvest::html_elements(result.rv, '.result')[-1])#Strip out header
      local.result <- local.result[!grepl('<p class=\"result\">\r\n</p>\n', local.result)]    #Strip out returns and form into list 
    }
  }
  
  return(local.result)
}

######################################################################################################################################
## Parse Taxon History Changes
######################################################################################################################################
tax_hist_changes <- function(changes) {
  ref.dat <- qdapRegex::ex_between(changes, left = "-- (",right = ")")
  if (is.na(ref.dat)) {
    ref.dat <- qdapRegex::ex_between(changes, left = "(",right = ")", include.markers = T)#separate reference data, include markers if needed
  }
  if (length(ref.dat[[1]]) > 1) { #When missing double hyphens with subfamily, as in floridana, Cichla
    ref.dat <- ref.dat[[1]][-1]
  }
  split.refs <- trimws(unlist(str_split(ref.dat, ",")))
  oriSplit <- split.refs
  
  startRefs <- str_extract(split.refs, "ref\\..*?\\>.*?\\</a")
  startRefs <- str_remove(startRefs, "\\<.*?\\>")
  startRefs <- str_remove(startRefs, "\\</a")
  startRefs <- str_remove(startRefs, "^ref\\. ")
  
  missingDataIndices <- sort(unique(c(which(!grepl(":", split.refs)), which(!grepl("\\[ref.*?\\]", split.refs))))) # Get positions of lines with issues
  toDeleteIndices <- c() # Storage for the lines that will need to be removed
  
  for (i in missingDataIndices) {
    if (i %in% toDeleteIndices | length(split.refs) == 1) {next} # Skip entries already marked for deletion
    
    if ((i+1) %in% missingDataIndices) { # If sequential entries are missing information, smash them together
      if ((i+2) %in% missingDataIndices) {
        split.refs[i] <- paste(split.refs[i], ", ", split.refs[i+1], ", ", split.refs[i+2], sep = "")
        toDeleteIndices <- c(toDeleteIndices, (i+1), (i+2))
      } else {
        split.refs[i] <- paste(split.refs[i], ", ", split.refs[i+1], sep = "")
        toDeleteIndices <- c(toDeleteIndices, (i+1))
      }
    } 
    
    else if (grepl(":", split.refs[i]) && !grepl("\\[ref.*?\\]", split.refs[i])) { # Handle cases that do not have a reference number
      split.refs[i] <- paste(split.refs[i], fixed("[ref.NA]"), sep = "")
    } 
    
    else {
      split.refs[i-1] <- paste(split.refs[i-1], ", ", split.refs[i], sep = "") # Handle cases when authors contain a comma
      toDeleteIndices <- c(toDeleteIndices, i)
    }
  }
  if(!is.null(toDeleteIndices)) {split.refs <- split.refs[-1*toDeleteIndices]} # Remove modified indices
  
  tbl_tempRef <- data.frame(matrix(nrow = length(split.refs), ncol = 3))
  colnames(tbl_tempRef) <- c("nom.ref", "nom.ref.no", "ref.order")
  for (i in seq_along(split.refs)) {
    if (grepl("^[0-9]+:", split.refs[i])) { # If no author, set nom.ref to NA
      tbl_tempRef[i, 1] <- NA
    } 
    else { # Otherwise find the author and year, stopping before page number
      nom.ref <- str_remove(split.refs[i], "\\[ref\\..*?\\]")
      nom.ref <- str_remove(nom.ref, ":.*")
      tbl_tempRef[i, 1] <- nom.ref
    }
    
    if (!grepl("\\[ref\\..*?\\]", split.refs[i])) { # If no reference, set nom.ref.no to NA
      tbl_tempRef[i, 2] <- NA
    }
    else { # Otherwise extract the reference
      nom.ref.no <- str_extract(split.refs[i], "\\[ref\\..*?\\]")
      nom.ref.no <- trimws(str_remove_all(nom.ref.no, "\\<a.*?\\>|\\<\\/a\\>|\\[|\\]|ref\\."))
      tbl_tempRef[i, 2] <- nom.ref.no
    }
    tbl_tempRef[i, 3] <- which(grepl(tbl_tempRef[i, 2], oriSplit))[1]
  }
  
  missingRefs <- data.frame(matrix(nrow = 0, ncol = 2))
  if(length(tbl_tempRef$nom.ref.no) != length(startRefs)) {
    missing <- c(setdiff(tbl_tempRef$nom.ref.no, startRefs), setdiff(startRefs, tbl_tempRef$nom.ref.no))
    for (i in seq_along(missing)) {
      if (!is.na(missing[i])) {
        refi <- which(grepl(missing[i], oriSplit))[1]
        nom.ref <- str_remove(str_extract(oriSplit[refi], "^.*?ref\\."), "ref\\.")
        nom.ref <- trimws(str_remove(nom.ref, "\\["))
        tbl_tempRef <- rbind(tbl_tempRef, c(nom.ref, missing[i], refi))
      }
    }
  }
  
  tbl_tempRef <- tbl_tempRef[order(tbl_tempRef$ref.order),]
  
  return(list(tbl_tempRef$nom.ref, tbl_tempRef$nom.ref.no))
}

#' Search for journals in the Catalog of Fishes
#' @param query Character vector to search for.
#' @param phrase Logical. Should query be passed as a quoted phrase (e.g. "Journal of Zoology"). Default is FALSE.
#' @param sleep.time Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.
#' @param verbose Logical. Should query progress be messaged to the screen? Default is TRUE.
#' @return Two column data frame. The first column contains the name of the query and the second column contains information on the journal.
#' @details This function returns information on the journals cited in the Eschmeyer's Catalog of Fishes.
#' @examplesIf interactive()
#' \donttest{
#' my.journals<-rcatfish_journals(query="Cichlid")
#' my.journals<-rcatfish_journals(query="Evolution")
#' }
#' @author Samuel R. Borstein
#' @references 
#' Fricke, R. & Eschmeyer, W.N. (Year Accessed). Eschmeyer’s Catalog of Fishes: Journals. https://researcharchive.calacademy.org/research/ichthyology/catalog/journals.asp
#' 
#' @export

rcatfish_journals<-function(query, phrase = FALSE,sleep.time = 10, verbose = TRUE){
  journal.dat <- data.frame(matrix(nrow = 0,ncol = 2),stringsAsFactors = FALSE)
  colnames(journal.dat) <- c("query","Journal")
  for(query.index in seq_along(query)){
    if (verbose == TRUE) {
      message(paste0("Now on query ",query.index, " of ", length(query), ", ",query[query.index]))
    }
    result <- RCurl::postForm("https://researcharchive.calacademy.org/research/ichthyology/catalog/journals.asp",
                            contains=ifelse(phrase == TRUE, shQuote(query, type = "cmd"), query), Submit="Search",style='POST',.encoding = "utf-8")#Perform search
    result.rv <- xml2::read_html(result)#parse results
    local.results <- as.character(rvest::html_elements(result.rv,".bodytext")[-1])#Strip out header
    strip.res <- StripFun(local.results)#Strip out html tags
    refs <- strsplit(strip.res,"\r\n\t\t\n")#split records into individual
    refs <- strsplit(refs[[1]],"\r\n\t\t\r\n  ")#split records into individual
    refs <- gsub(pattern = "\r\n\\[.*].*\r\n\n",replacement = '',x = refs)#Kill of some extra text
    refs<-gsub("(?<=[\\s])\\s*|^\\s+|\\s+$", "", refs, perl=TRUE)
    current.refs<-data.frame(rep(query[query.index],length(refs)),refs,stringsAsFactors = FALSE)
    journal.dat <- rbind.data.frame(journal.dat,current.refs,stringsAsFactors = FALSE)
    if (length(query)>1) {
      Sys.sleep(sleep.time)#Sleep between calls for non-overload if query is longer than 1
    }
  }
  colnames(journal.dat)<-c("Query","Journals")
  return(journal.dat)
}

#Test that rcatfish_journals returns proper journal information


test_that("Test that rcatfish_journals returns proper journal information",{
  journal.hit <-try(rcatfish_journals(query = "Buntbarsche"))
  if ("try-error"%in%class(journal.hit)) {
    skip("could not connect to remote database")
    }else{
      expect_identical(as.numeric(grep("American Cichlid Association; 1966 up to the present / Journal of the American cichlid Association.",as.vector(journal.hit$Journals))), 1)
    }
  }
)

.pkgglobalenv <- new.env(parent=emptyenv())
.onAttach <- function(libname, pkgname) {
  homeCOF <- base::readLines('https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp',warn = FALSE)
  versionLine<-grep("version of",homeCOF)
  cof_version<-rvest::html_text(xml2::read_html(homeCOF[versionLine]))
  cof_version<-strsplit(cof_version," - |  ")[[1]][2]
  packageStartupMessage("Welcome to rcatfish: Access to the California Academy of Sciences Eschmeyer's Catalog of Fishes") 
  packageStartupMessage(paste0("Catalog of Fishes ", cof_version))
  packageStartupMessage(paste0("If you use this package please cite both:\n\nThis R package: Borstein, S.R., Dominy, B.E. & O'Meara, B.C.(2026) rcatifsh: An R interface to the California Academy of Sciences Eschmeyer's Catalog of Fishes. Available at:https://github.com/sborstein/rcatfish.\n\n", paste0("Catalog of Fishes: Fricke, R., Eschmeyer, W.N. & van der Laan, R. ", lubridate::year(Sys.Date()), ". Catalog of Fishes: Genera, Species, References. Available at: https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp. Electronic version accessed ",format(Sys.Date(), "%d %m %Y"),".")))
  assign("cof_version", cof_version, envir=.pkgglobalenv)
}

#' Find number of available fish names, number of valid species, and number of descriptions in the last 10 years by family or subfamily
#' @param query A character vector of taxa to search for. For users wishing to obtain the information for all fishes please query "Totals".
#' @param verbose Logical. Should query progress be messaged to the screen? Default is TRUE.
#' @return A list with each element containing a data frame for each query. List elements are named by the queried taxa. 
#' The list contains information on the number of genera and species available, valid, and described in the last 10 years. An itemized list describing the contents in the columns of the data frame(s) returned is described below.
#' #' \itemize{
#'   \item Class - Character. Taxonomic class.
#'   \item Order - Character. Taxonomic order.
#'   \item Family - Character. Taxonomic family
#'   \item Subfamily - Character. Taxonomic subfamily.
#'   \item Available.Genera - Integer. Number of available genera for the taxonomic rank.
#'   \item Valid.Genera - Integer. Number of valid genera for the taxonomic rank.
#'   \item Genera.Last.Ten.Years - Integer. Number of genera described over the past ten years for the taxonomic rank.
#'   \item Available.Species - Integer. Number of available species for the taxonomic rank.
#'   \item Valid.Species - Integer. Number of valid species for the taxonomic rank.
#'   \item Species.Last.Ten.Years - Integer. Number of species described over the past ten years for the taxonomic rank.
#'   }
#' @description This function retrieves and parses data on the number of genera and species per family from the 
#' Eschmeyer's Catalog of Fishes. It returns a table containing the number of genera and species that are available, valid,
#' and described in the last 10 years in its columns. Each row represents either a class, order, family, or subfamily, so 
#' some columns for higher taxonomic groups may have NA for lower taxonomic groups (e.g. if you Search for Cypriniformes, the row 
#' containing Cypriniformes will have NA for family and subfamily as it is above those taxonomic levels).
#' 
#' Given the function reads in the page containing the table, for speed, it is highly recommended that if a user wishes to obtain
#' information on numerous orders, families, or subfamilies that they pass them in as a single query. To do this, see the last example.
#' @author Samuel R. Borstein
#' @references 
#' Fricke, R., Eschmeyer, W.N. & Fong J.D. (Year Accessed). Eschmeyer’s Catalog of Fishes: Species by family/subfamily in the Catalog of Fishes. https://researcharchive.calacademy.org/research/ichthyology/catalog/SpeciesByFamily.asp.
#' @examplesIf interactive()
#' \donttest{
#' #Search for the number of described Cypriniformes
#' MySearch <- rcatfish_species_by(query = "Cypriniformes") 
#' #Obtain the total number of genera and species available, valid, and described in the last 10 years
#' CofF_totals <- rcatfish_species_by(query = "Totals")
#' #Search for more than one family
#' MySearch <- rcatfish_species_by(query = c("Cichlidae","Embiotocidae"))
#' }
#' @export

rcatfish_species_by <- function(query, verbose = TRUE) {
  rcatfish_taxa <- GetTaxonTable()
  TaxaSummary <- list()
  for (query.index in seq_along(query)){
    if (verbose == TRUE) {
      message(paste0("Now on query ",query.index, " of ", length(query),", ",query[query.index]))
    }
    if(query[query.index] == "Totals"){
      TotalHit <- nrow(rcatfish_taxa)
      Totals <- rcatfish_taxa[TotalHit,]
      Totals[1,2:4] <- "Totals"
      TaxaSummary[[query.index]] <- data.frame(Totals, stringsAsFactors=FALSE)
      names(TaxaSummary)[query.index] <- query[query.index]
      }else{
      found <- which(apply(X = rcatfish_taxa, MARGIN = 2, FUN = is.element, el = query[query.index])==TRUE)[1]#Find the taxon in table
      if(is.na(found)){
        warning(paste0("No results found for supplied taxon ",query[query.index]))
      }else{
        Focal.Tax <- which(as.character(rcatfish_taxa[,found])==query[query.index])#Find Children of taxa
        Focal.Tax.Dat <- rcatfish_taxa[Focal.Tax,]#Subset children of taxa
        rownames(Focal.Tax.Dat) <- nrow(Focal.Tax)#renumber rows
        Taxa2Sum <- suppressWarnings(which(is.na(as.numeric(Focal.Tax.Dat$`Valid Genera`))))#Find group to summarize
        SubFam2Kill <- which(!is.na(Focal.Tax.Dat$Subfamily))
        if(length(SubFam2Kill) > 0){
          Fams2Sum <- Focal.Tax.Dat[-SubFam2Kill,]#Kill off subfamilies, which are already summed at family level
          Fams2Sum <- Fams2Sum[!is.na(Fams2Sum$Family),]
        }else{
          Fams2Sum <- Focal.Tax.Dat[!is.na(Focal.Tax.Dat$Family),]
        }
        MissingClass <- Focal.Tax.Dat[which(is.na(Focal.Tax.Dat$Order) & is.na(Focal.Tax.Dat$Family)),]
        MissingOrder <-  Focal.Tax.Dat[which(!is.na(Focal.Tax.Dat$Order) & is.na(Focal.Tax.Dat$Family)),]
        #Fix Duplicate Name Issues
        if(nrow(MissingOrder) > 0){
          for(order.index in 1:length(unique(MissingOrder$Order))){
            Dat2Store.Order <- match(MissingOrder$Order[order.index], Focal.Tax.Dat$Order)
            Current.Order <- Fams2Sum[Fams2Sum$Order == MissingOrder$Order[order.index],]
            Focal.Tax.Dat[Dat2Store.Order,5:10] <- colSums(Current.Order[,5:10])
          }
        }
        if(nrow(MissingClass) > 0){
          for(class.index in 1:length(unique(MissingClass$Class))){
            Dat2Store.Class <- match(MissingClass$Class[class.index], Focal.Tax.Dat$Class)
            Current.Class <- Fams2Sum[Fams2Sum$Class == MissingClass$Class[class.index],]
            Focal.Tax.Dat[Dat2Store.Class,5:10] <- colSums(Current.Class[,5:10])
          }
        }
        TaxaSummary[[query.index]] <- data.frame(Focal.Tax.Dat, stringsAsFactors=FALSE)
        names(TaxaSummary)[query.index] <- query[query.index]
      }
    }
    }
  return(TaxaSummary)
}


#' Get the table with species numbers by taxonomy
#' 
#' @return Object of class data.frame of taxonomy and number of described/valid species
#' @author Samuel R. Borstein
#' @noRd

GetTaxonTable <- function() {
  webpage <- xml2::read_html("https://researcharchive.calacademy.org/research/ichthyology/catalog/SpeciesByFamily.asp")
  tbls <- rvest::html_elements(webpage, "table")#capture table
  speciesTable<- rvest::html_table(x = tbls[5])[[1]]
  classes <- c("Totals","Myxini","Petromyzonti","Elasmobranchii","Holocephali","Cladistii","Actinopteri","Coelacanthi","Dipneusti")#Make vector of classes to find to keep
  speciesTable$Class[!speciesTable$Class%in%classes]<- NA#kill off non-classes in class column
  speciesTable$Order[which(!grepl("iformes",speciesTable$Order))] <- NA#kill off non-orders in order column
  speciesTable$Family[which(!grepl("idae",speciesTable$Family))] <- NA#kill off non-families in family column
  speciesTable$Subfamily[which(!grepl("inae",speciesTable$Subfamily))] <- NA#kill off non-subfamily in subfamily column
  #colnames(speciesTable) <- speciesTable[1,]#Rename columns into useful variables
  #speciesTable <- speciesTable[-1,]#Kill header lines
  #filled<-as.data.frame(apply(speciesTable, 2, function(x) gsub("^$|^ $", NA, x)),stringsAsFactors = FALSE)#Replace empty with NA
  speciesTable <- tidyr::fill(speciesTable, "Family", .direction = c("down"))#Fill in NA with the family/order
  speciesTable$Family[is.na(speciesTable$`Available Genera`)]<-NA
  speciesTable <- tidyr::fill(speciesTable, "Order", .direction = c("down"))#Fill in NA with the family/order
  speciesTable$Order[which(!is.na(speciesTable$Class)&is.na(speciesTable$`Available Genera`))]<-NA
  speciesTable <- tidyr::fill(speciesTable, "Class", .direction = c("down"))#Fill in NA with the family/order
  speciesTable[grep("Totals",speciesTable$Class),2:4]<-NA#for Totals, make it only Totals
  speciesTable <- as.data.frame(speciesTable)
  return(speciesTable)
}

#Test that rcatfish_species_by returns proper information


test_that("Test that rcatfish_species_by returns proper information",{
  species.hit <-try(rcatfish_species_by(c("Ceratodontiformes", "Coelacanthiformes")))
  if ("try-error"%in%class(species.hit)) {
    skip("could not connect to remote database")
    }else{
      expect_identical(species.hit$Ceratodontiformes$Valid.Genera[1], 3)#Check names are correct
      expect_equal(species.hit$Coelacanthiformes$Valid.Species[1], 2)#Check it say update and has month and year
    }
  }
)

#' Access catalog changes and additions
#' 
#' @description This function is used to access the Eschmeyer's Catalog of Fishes Summary of Changes and Additions.
#' 
#' @param changes Logical. Should the function return the table of changes to the catalog? Default is TRUE.
#' @param author.changes Logical. Should the function return the table of author and year changes to the catalog? Default is TRUE.
#' @param spell.changes Logical. Should the function return the table of spelling changes to the catalog? Default is TRUE.
#' @param added.genera Logical. Should the function return genera added to the catalog? Default is TRUE.
#' @param added.species Logical. Should the function return species added to the catalog? Default is TRUE.
#' @details This function displays the information on the "Changes and Additions" tab of Eschmeyer's Catalog of Fishes. The catalog updates this information monthly with changes to the catalog, new genus information, and new species information. Each updates removes the last month's information, so data obtained through this function is only representative of the changes and additions made since the previous update.
#' @return A named list. Names include UpdateDate, Changes, AuthorshipChanges, AddedGenera, and AddedSpecies. UpdateDate will always be returned and is a character string of the date the catalog was updated. Changes, AddedGenera, and AddedSpecies summarize changes to the catalog, changes to authorship information, genera added to the catalog, and species added to the catalog respectively. The elements returned are based on which parameters are set to TRUE. An itemized list describing the contents in the columns of the data returned is described below.
#' \itemize{
#'   \item Original.Taxon - Character. The original taxonomic name in the database.
#'   \item Was - Character. Previous status of the taxon in the database.
#'   \item Previous.Taxon - Character. Previous taxonomic name in the database.
#'   \item Now - Character. Current status of the taxon in the database following the update.
#'   \item Current.Taxon - Character. Current taxonomic name in the database for the taxon following the update.
#'   \item Taxon - Character. Taxon (of any rank) whose citation was updated.
#'   \item Previous.Author - Character. Authorship information of the citation before the update.
#'   \item Current.Author - Character. Authorship information of the citation following the update.
#'   \item Species.Name - Character. Current taxonomic name in the database for the taxon following the update.
#'   \item Genus.Name - Character. Name of the new genus added to the database following the update.
#'   \item Authors - Character. Authorship for new taxonomic names added to the database following the update.
#'   \item Year   - Character. Year for new taxonomic names added to the database following the update.
#'   \item Region - Character. Region new taxa added to the database are known to occur in.
#'   \item Was.Spelled - Character. Previous spelling of the taxon in the database.
#'   \item Now.Spelled - Character. Revised spelling of the taxon in the database.
#'   }
#' @examplesIf interactive()
#' \donttest{
#' # return all available data
#' myList <- rcatfish_updates(changes = TRUE, added.genera = TRUE, added.species = TRUE)
#' }
#' @author Brandon E. Dominy, Samuel R. Borstein
#' @references 
#' Fricke, R., van der Laan, R. & Fong, J.D. (Year Accessed). Eschmeyer’s Catalog of Fishes: Changes and Additions. https://researcharchive.calacademy.org/research/ichthyology/catalog/ChangeSummary.asp.
#' @export
#' @importFrom xml2 read_html
#' @importFrom magrittr %>%
#' @import rvest
#' @import stringr

rcatfish_updates <- function(changes = TRUE, author.changes = TRUE, spell.changes = TRUE, added.genera = TRUE, added.species = TRUE) {
  
  # returns an error if parameters are not type 'logical'
  # if(!is.logical(c(changes, author.changes, spell.changes, added.genera, added.species))){
  #   stop("'changes', 'author.changes', 'added.genera', and 'added.species' must be provided as logical values")
  #   
  # }
  if(any(c(is.logical(changes), is.logical(author.changes), is.logical(spell.changes),is.logical(added.genera),is.logical(added.species))==FALSE)){
    stop("'changes', 'author.changes', 'added.genera', and 'added.species' must be provided as logical values")
  }

  ##############################################################################
  
  ########################### Accessing Catalog HTML ###########################
  tryCatch(expr = {url <- "https://researcharchive.calacademy.org/research/ichthyology/catalog/ChangeSummary.asp" # Link to page in catalog of fish
  page.html <- xml2::read_html(url) # Read in html from link
  
  
  ###################### Retrieve Date of Catalog Update #######################        
  page.paras <- rvest::html_elements(page.html, 'p') %>% # Extract paragraphs into vector
    rvest::html_text()  # Get paragraph text for each item in vector
  
  update.index <- grep(pattern = "update", page.paras) # Retrieve index of item containing update information
  
  updatedOn <- page.paras[update.index] %>% # Retrieve only vector item containing 'update' text
    str_extract(string = , pattern = "update of .*\\d{4}") %>% # Separate out all text except 'update of [date]' from vector
    str_replace(string = , pattern = 'update of', replacement = 'Updated on') %>% # Reformat text for user readability
    trimws() # Trim white space from date text, assign to character vector for output
  
  
  ######################## Retrieve Changes and Authorship Updates to Catalog #########################         
  page.tbls <- rvest::html_table(page.html) # Scrape data from site tables
  changesData <- as.data.frame(page.tbls[4]) # Convert table of changes (always fourth table from site) to dataframe for output
  changesAuthor <- as.data.frame(page.tbls[5]) # Convert table of authorship changes (always fifth table from site) to dataframe for output
  changesSpelling <- as.data.frame(page.tbls[6]) # Convert table of authorship changes (always fifth table from site) to dataframe for output
  
  if (length(changesAuthor) == 0) {author.changes <- FALSE} # If there are no authorship changes, do not return author data
  if (length(changesSpelling) == 0) {spell.changes <- FALSE} # If there are no authorship changes, do not return author data
  
  table.html <- rvest::html_elements(page.html, 'table') # Get the raw html of all tables on the page
  updates.html <- table.html[4] # Pull the raw html of specifically the changes table
  changes.html <- as.character(rvest::html_elements(updates.html, ".changes")) # Retrieve individual table cells as raw html
  changes.html <- changes.html[6:length(changes.html)] # Strip out header row
  changes.html <- str_remove_all(changes.html, "<td.*?>|</td>|<a.*?>|</a>|\\\n|<i>") # Remove html tags
  changes.html <- str_replace_all(changes.html, "&amp;", "&") # Fix ampersand issues
  changes.indices <- 1:length(changes.html) # Get vector of total cells indices
  origTax <- which((changes.indices-1) %% 5 == 0) # Get vector of indices of all orginal taxon fields
  was <- which((changes.indices-2) %% 5 == 0) # Get vector of indices of all was fields 
  prevTax <- which((changes.indices-3) %% 5 == 0) # Get vector of indices of all previous taxon fields
  now <- which((changes.indices-4) %% 5 == 0) # Get vector of indices of all now fields
  currTax <- which(changes.indices %% 5 == 0) # Get vector of indices of all current taxon fields
  split.previous <- str_split(changes.html[prevTax], "</i>") # Separate authors of previous taxa
  split.current <- str_split(changes.html[currTax], "</i>") # Separate authors of current taxa
  tbl.changes.split.author <- as.data.frame(matrix(nrow = nrow(changesData), ncol = 7)) # Create table to store data
  colnames(tbl.changes.split.author) <- c("Original.Taxon", "Was", "Previous.Taxon", "Previous.Author", "Now", "Current.Taxon", "Current.Author")
  for (i in 1:nrow(tbl.changes.split.author)) { # Fill in output table
    tbl.changes.split.author[i, 1] <- str_remove(changes.html[origTax[i]], "</i>")
    tbl.changes.split.author[i, 2] <- changes.html[was[i]]
    tbl.changes.split.author[i, 3:4] <- split.previous[[i]]
    tbl.changes.split.author[i, 5] <- changes.html[now[i]]
    tbl.changes.split.author[i, 6:7] <- split.current[[i]]
  }
  
  
  ################## Retrieve Genera/Species Added to Catalog ##################      
  page.links <- rvest::html_elements(page.html, "a") # Scrape all links on site
  page.linkText <- rvest::html_text2(page.links) # Strip links of tags (now only link text)
  link.break.index <- min(which(page.linkText == changesData[nrow(changesData), 1])) # Find which position in link text vector matches last entry in table of changes (all links after this point are added genera/species)
  added.data <- page.linkText[-1:-link.break.index] # Create vector from link text using only positions of added genera and species
  
  
  page.text <- page.html %>% # Create variable page_text, start pipe with html data
    rvest::html_children() %>% # Creates nodeset with two objects: data stored in <head> tags (metadata) and data in <body> tags (visisble content)
    rvest::html_text2() %>% # Convert all data into text, without tags. Creates vector with two objects: head data as text, body data as text
    strsplit( , split = "Added Genera") %>% # Split text at "Added Genera" label. Returns list with 2 items: [1] head data as text, [2] contains two objects: [1] body data before "Added Genera", [2] body data after
    unlist() %>% # Split list into vector length 3: [1] head data, [2] data before "Added Genera", [3] data after "Added Genera"
    strsplit( , split = "Added Species") %>% # Perform same split as above, this time on "Added Species". Returns list of length 3, with the third object split into [1] data before "Added Species", [2] data after
    unlist() # Split list into vector length 4: [1] head data, [2] data before "Added Genera", [3] data between "Added Genera" and "Added Species", [4] data after "Added Species"
  # Ends up with vector page_text of length 4. [1] contains header data, [2] contains change table data, [3] contains Added Genera data, [4] contains Added Species data
  
  gen.data <- page.text[3] # Data for added genera
  spec.data <- page.text[4] # Data for added species
  
  .dataFix <- function(criteria, TaxData) { # Internal helper function for processing genera data and species data
    splitData <- strsplit(TaxData, split = '\\\n') %>% # split input on newline character '\n'
      unlist()
    indices <- unlist(sapply(criteria, grep, splitData, fixed = T))#sapply to grep across all names and find positions
    dataValues <- splitData[indices]
  }
  
  frmt.gen.data <- .dataFix(criteria = added.data[grep(" ",added.data,invert = TRUE, fixed = TRUE)], TaxData = gen.data) # Formatted genera data
  frmt.spec.data <- .dataFix(criteria = added.data[grep(" ",added.data, invert = FALSE, fixed = TRUE)], TaxData = spec.data) # Formatted species data
  
  newGenera <- as.data.frame(matrix(nrow = 0, ncol = 3)) # Create matrix to store scraped genus data
  
  if (length(frmt.gen.data) == 0) { # If there are no added genera, fill matrix with NA values
    name <- author <- year <- NA
    
    newGenera <- cbind.data.frame(name, author, year)
  } else { # If there are added genera, fill matrix with proper data
    frmt.gen.data <- str_replace_all(string = frmt.gen.data, pattern = '\\\r', replacement = '') # Remove carriage return characters at start of genera (this is specific to genus table)
    
    name <- trimws(stringr::str_extract(pattern = '^[A-Z][a-z]* ', string = frmt.gen.data))
    
    year <- str_extract(pattern = '\\d{4}', string = frmt.gen.data)
    
    authorStart <- stringr::str_locate(pattern = name, string = frmt.gen.data)
    authorEnd <- stringr::str_locate(pattern = year, string = frmt.gen.data)
    author <- stringr::str_sub(string = frmt.gen.data, start = authorStart[,2]+1, end = authorEnd[,1]-1) %>%
      trimws()
    
    newGenera <- cbind.data.frame(name, author, as.numeric(year)) 
  }
  
  colnames(newGenera) <- c('Genus.Name', 'Authors', 'Year') # Set matrix column names
  
  
  newSpecies <- as.data.frame(matrix(nrow = 0, ncol = 4)) # Create matrix to store scraped species data
  
  if (length(frmt.spec.data) == 0) { # If there are no added species, fill matrix with NA values
    name <- author <- year <- region <- NA
    
    newSpecies <- cbind.data.frame(name, author, as.numeric(year), region)
  } else { # If there are added species, fill matrix with proper data
    frmt.spec.data <- str_replace_all(string = frmt.spec.data, pattern = '\\\r', replacement = '') # Remove carriage return characters at start of genera (this is specific to genus table)
    
    name <- name <- trimws(stringr::str_extract(pattern = '^[A-Z][a-z]* [a-z]* [a-z]*|^[A-Z][a-z]* \\([A-Z][a-z]*\\)* [a-z]*', string = frmt.spec.data))
    
    year <- stringr::str_extract(pattern = '\\d{4}', string = frmt.spec.data)
    
    authorStart <- stringr::str_locate(pattern = fixed(name), string = frmt.spec.data)
    authorEnd <- stringr::str_locate(pattern = year, string = frmt.spec.data)
    author <- stringr::str_sub(string = frmt.spec.data, start = authorStart[,2]+1, end = authorEnd[,1]-1) %>%
      trimws()
    
    regionStart <- stringr::str_locate(pattern = year, string = frmt.spec.data)
    regionEnd <- stringr::str_length(frmt.spec.data)
    region <- stringr::str_sub(string = frmt.spec.data, start = regionStart[,2]+2, end = regionEnd-1) %>%
      trimws()
    for (i in seq_along(region)) { # Replace missing regions with NA values
      if (region[i] == '') {
        region[i] <- NA
      }
    }
    
    newSpecies <- cbind.data.frame(name, author, as.numeric(year), region)
  }
  colnames(newSpecies) <- c('Species.Name', 'Authors', 'Year', 'Region') # Set matrix column names
  
  
  #############################  Parse Function Output  #############################
  outputList <- list(updatedOn) # Always add date of update to output list
  names(outputList) <- "UpdateDate"
  
  if (changes) { # If changes == TRUE, add catalog changes to output list
    OutputNames <- c(names(outputList),"Changes")
    outputList <- append(outputList, values = list(tbl.changes.split.author))
    names(outputList) <- OutputNames
  }
  if (author.changes) {
    OutputNames <- c(names(outputList),"AuthorshipChanges")
    outputList <- append(outputList, values = list(changesAuthor))
    names(outputList) <- OutputNames
  }
  if (spell.changes) {
    OutputNames <- c(names(outputList),"SpellingChanges")
    outputList <- append(outputList, values = list(changesSpelling))
    names(outputList) <- OutputNames
  }
  if (added.genera) { # If added.genera == TRUE, add new genera to output list
    OutputNames <- c(names(outputList),"AddedGenera")
    outputList <- append(outputList, values = list(newGenera))
    names(outputList) <- OutputNames
  }
  if (added.species) { # If added.genera == TRUE, add new species to output list
    OutputNames <- c(names(outputList),"AddedSpecies")
    outputList <- append(outputList, values = list(newSpecies))
    names(outputList) <- OutputNames
    
  }
  
  return(outputList) # Return output list
  }, error = function(cond) { # Error handling
    if(!curl::has_internet()) { # Error for bad internet connection
      message(paste('Please check your internet connection and try again'))
    } else { # Generic error
      message(paste0("Undefined Error: ", cond))
      
      return(NA)
    }
  }
  )
}

#Test that rcatfish_updates returns proper information


test_that("Test that rcatfish_updates returns proper definitions",{
  updates.hit <-try(rcatfish_updates())
  if ("try-error"%in%class(updates.hit)) {
    skip("could not connect to remote database")
    }else{
      expect_in(names(updates.hit), c("UpdateDate", "Changes", "AuthorshipChanges","AddedGenera", "AddedSpecies", "SpellingChanges"))#Check names are correct
      has_valid_date <- grepl("Updated on \\d{1,2} [a-zA-Z.]+[[:space:]]\\d{4}", updates.hit$UpdateDate)#Check it say update and has month and year
      if (!has_valid_date) {
        skip("Skipping test: Update date does not match the expected date pattern")
      }
      expect_true(has_valid_date)
    }
  }
)

#' Search for genera and species in the Eschmeyer's Catalog of Fishes
#' 
#' @param query Character or Character Vector containing the name or names of the taxon to search. Note, you can not mix common and scientific names as a query.
#' @param type Character either "Genus" or "Species" to search for genera and species respectively.
#' Note that only one of these options can be chosen.
#' @param unavailable Logical. Should the query be run with unavailable names include? Default is FALSE.
#' @param taxon.history Should a detailed history of taxonomic changes per taxa be returned (i.e. synonymization, raised to validity, authority, etc.). Default is FALSE.
#' @param resolve Logical. If a match for the query isn't found, should an attempt be made to resolve the name using taxize?
#' @param sleep.time Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.
#' @param phrase Logical. Should query be passed as a quoted phrase (e.g. "Synonym of Cyprinus carpio"). Default is FALSE.
#' @param verbose Logical. Should query progress be messaged to the screen? Default is TRUE.
#' @param common.name Is the query a common name? Common names will be converted to scientific names for searching through rfishbase. Note, you can not mix common and scientific names as a query. This likely will only work for species searches. Default is FALSE.
#' @param language Language to perform common name search. Default is English.
#' @details This function searches for genera or species in the Catalog of Fishes and returns its
#' valid status, synonyms, and taxonomic history as well as reference numbers for the authority of
#' the citations. By default, the function returns basic information on a taxon, such as who described it, its current taxonomic status, type locality,
#' gender of the name, etc. If users choose taxon.history = TRUE, a detailed list of taxonomic information regarding nomenclature acts associated with the taxa is also returned.
#' Note that the function will take longer to run, sometimes twice as long if taxon.history = TRUE.
#' 
#' One problem a user may encounter using the Catalog of Fishes website is that the input taxon name must match directly to a term in the database 
#' or the database will not return any information. While this remains true using this package, users can attempt to resolve names by setting resolve = TRUE.
#' When resolve = TRUE, rcatfish_search will use the Global Names Verifier (GNverifier) in an attempt to resolve the name, which will then be passed to downstream function calls.
#' This is meant to be useful, but we recommend using this option be cautious about what the GNverifier returns. A message will print to the screen notifying you what 
#' name the GNverifier resolved to be the best match and will be used, though we strongly recommend users check the resolved name does not deviate from their expectations (i.e. a homonym or similar name for a different group is not returned).
#' @return Data frames. If taxon.history = TRUE, a list of two data frames. In this case, the first data frame TaxonSummary contains information on the description and current
#' status of the taxa in the query, references to descriptions, and information on the type locality, types, family/subfamily, distribution, and habitat for species and type species
#' gender, status, and authorities for genera searches. The second data frame, TaxonHistory contains detailed information on the taxonomic history 
#' of a taxon, such as which authorities have viewed it as a synonym or valid since its description. An itemized list describing the contents in the columns of the data returned is described below.
#' \itemize{
#'   \item Query - Character. The submitted query.
#'   \item Nominal Taxa - Character. Nominal taxonomic names.
#'   \item Author - Character. Authorship of the species/Genus description.
#'   \item DescriptionRef - Character. Eschmeyer Catalog of Fishes reference number for the genus or species description.
#'   \item DescriptionYear - Numeric. Year in which taxon was described.
#'   \item Status - Character. Current status of the nominal taxon.
#'   \item CurrentNomenclature - Character. Currently recognized taxonomic name of the taxon.
#'   \item CurrentAuthority - Character. Current authority for the valid name of the taxon.
#'   \item Holotype - Character. Catalog number of the holotype.
#'   \item Paratype - Character. Catalog number(s) of the paratypes.
#'   \item Lectotype - Character. Catalog number of the lectotype.
#'   \item Paralectotype - Character. Catalog number(s) of the paralectotype.
#'   \item Neotype - Character. Catalog number of the neotype.
#'   \item Syntype - Character. Catalog number(s) of the syntypes.
#'   \item NoTypes - Character. Specifies entries with currently no known types.
#'   \item TypeLocality - Character. Type locality of the taxon.
#'   \item Family - Character. Family the taxon belongs to.
#'   \item Subfamily - Character. Subfamily the taxon belongs to.
#'   \item Distribution - Character. Distribution of the species.
#'   \item Fresh - Numeric. Binary presence (1) or absence (0) in freshwater.
#'   \item Brackish - Numeric. Binary presence (1) or absence (0) in brackish water.
#'   \item Marine - Numeric. Binary presence (1) or absence (0) in marine water.
#'   \item IUCNYear - Numeric. Year in which IUCN status was assessed.
#'   \item IUCNStatus - Character. Status in the IUCN list of threatened species.
#'   \item NomenclatureNotes - Character. Descriptive tags identifying status as a homonym, hybrid, nomen protectum, etc.
#'   \item TypeSpecies - Character. Type Species of the genus.
#'   \item Gender - Character. Gender of the taxon.
#'   \item TypeBy - Character. Type designation.
#'   \item Notes - Character. Any notes related to the taxon or taxonomic history (e.g. treated as a subspecies, availability of name, authorship issues, etc.).
#'   \item AsSubgenus - Character. If the taxon was described as a subgenus of another genus, provides information on the genus.
#'   \item Infrasubspecific - Character. Infrasubspecific designation if it exists. 
#'   \item Miscellaneous - Character. Additional information not captured in other columns.
#'   \item ResultCode - Character. A unique identifier for each nominal taxa, useful for linking table rows when homonyms are present.
#' }
#' @examplesIf interactive()
#' #Note that for Windows OS, OpenSSL must be used as a backend for curl. 
#' #Please see vignette on how to do this with vignette('rcatfish').
#' 
#' #Search for Abactochromis and return taxon history
#' 
#' if((.Platform$OS.type == "windows") & (grepl(pattern = "\\(OpenSSL", 
#'   curl::curl_version()$ssl_version) == TRUE)){
#' cat("openSSL backend for curl is required for the Windows version of this package, but it not 
#' detected as being active. Please see the vignette on how to configure curl with openSSL for this
#' function to work. You can access the vignette with the following: vignette('rcatfish').")
#' }else{
#' my.search <- rcatfish_search(type = "Species", query = "Abactochromis", 
#'   taxon.history = FALSE, resolve = FALSE, sleep.time = 0)
#' }
#' 
#' #Search for the genera Astatheros and Gasteropelecidae
#' \donttest{
#' if((.Platform$OS.type == "windows") & (grepl(pattern = "\\(OpenSSL", 
#'   curl::curl_version()$ssl_version) == TRUE)){
#' cat("openSSL backend for curl is required for the Windows version of this package, but it not 
#' detected as being active. Please see the vignette on how to configure curl with openSSL for this
#' function to work. You can access the vignette with the following: vignette('rcatfish').")
#' }else{
#'   my.search <- rcatfish_search(type = "Genus", query = c("Astatheros","Gasteropelecidae"), 
#'     taxon.history = TRUE, resolve = FALSE, sleep.time = 10)
#'   }
#' 
#' #Perform a species search for two different taxa
#' 
#' if((.Platform$OS.type == "windows") & (grepl(pattern = "\\(OpenSSL", 
#'   curl::curl_version()$ssl_version) == TRUE)){
#' cat("openSSL backend for curl is required for the Windows version of this package, but it not 
#' detected as being active. Please see the vignette on how to configure curl with openSSL for this
#' function to work. You can access the vignette with the following: vignette('rcatfish').")
#' }else{
#' my.search<-rcatfish_search(type = "Species", query = c("Ctenopharynx",
#'   "Pseudocrenilabrus multicolor victoriae"), taxon.history = TRUE, resolve = FALSE, 
#'   sleep.time = 10)
#'   }
#' }
#' 
#' @references 
#' Fricke, R., Eschmeyer, W.N. & van der Laan, R. (Year Accessed). Eschmeyer's Catalog of Fishes: Genera, Species, References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp
#' 
#' Mozzherin, D. (2025). GNverifier -- a reconciler and resolver of scientific names against more than 100 data sources. (v1.3.0). Zenodo. https://doi.org/10.5281/zenodo.17245658
#' @author Samuel R. Borstein, Brandon E. Dominy, Brian C. O'Meara 
#' @export

rcatfish_search<-function(query, type, unavailable = FALSE, taxon.history = FALSE, resolve = FALSE, sleep.time = 10, phrase = FALSE, verbose = TRUE, common.name = FALSE, language = "English"){
  if(length(type) > 1){#Check length of type. Must be 1.
    stop("Length of argument type must be 1")
  }
  
  stopifnot("Argument type must be either Genus or Species" = type %in% c("Species", "Genus", "species", "genus"))#Error if type is not a valid option
  type <- paste(toupper(substr(type, 1, 1)), substr(type, 2, nchar(type)), sep="")#force lower case first letter to upper so it works.
  
  if(type != "Species" && common.name == TRUE) {
    stop("Common names can only be searched by species")
  }
  
  if(type == "Species") {
    output <- rcatfish_search_species(query, unavailable, taxon.history, resolve, sleep.time, phrase, verbose, common.name, language)
  } else if (type == "Genus") {
    output <- rcatfish_search_genus(query, unavailable, taxon.history, resolve, sleep.time, phrase, verbose)
  }  
  return(output)
}

####################################################################################
####################################################################################
####################################################################################
#
#  rcatfish_search_species
#
####################################################################################
####################################################################################
####################################################################################

rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history = FALSE, resolve = FALSE, sleep.time = 10, phrase = FALSE, verbose = TRUE, common.name = FALSE, language = "English") {
  ##### Check Arguments for Parameters #####
  
  ##### Check that Windows is Using OpenSSL ##### 
  if((.Platform$OS.type == "windows") & (grepl(pattern = "\\(OpenSSL", curl::curl_version()$ssl_version) == TRUE)){
    stop("openSSL backend for curl is required for the Windows version of this package, but is not detected as being active. Please see the vignette on how to configure curl with openSSL for this function to work. You can access the vignette with the following: vignette('rcatfish')")
  }
  
  ##### Retrieve Scientific Names from Common Names #####
  if (common.name == T) {
    all.common.names <- as.data.frame(GetSci(query = query, language = language))
    colnames(all.common.names) <- c("Query", "Species", "CommonName")
    query <- unique(all.common.names$Species)
    if(length(query)==0){
      stop("No scientific names found for common name(s) in query")
    }
  }
  
  sleep.time <- ifelse(length(query)>1, sleep.time, 0) # Set sleep time for queries longer than 1
  
  #Make Tables to store results
  tax_sum_columns <- c("Query",	"NominalTaxa",	"Author",	"DescriptionRef", "RefPage", "DescriptionYear", "Status", "CurrentNomenclature", "CurrentAuthority",	"Holotype",	"Paratype",	"Lectotype",	"Paralectotype",	"Neotype",	"Syntype", "NoTypes", "TypeLocality",	"Family","Subfamily",	"Distribution", "Fresh","Brackish", "Marine", "IUCNYear", "IUCNStatus", "NomenclatureNotes", "Infrasubspecific", "Miscellaneous", "ResultCode")
  all.tax.sum <- data.frame(matrix(nrow = 0, ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)
  colnames(all.tax.sum) <- tax_sum_columns
  
  if (taxon.history == T) {
    tax_hist_columns <- c("Query","NominalTaxa","Status","RecognizedNomenclature","Authority","RefNo","Notes", "ResultCode")
    all.taxon.history <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns)))
    colnames(all.taxon.history) <- tax_hist_columns 
  }
  
  for (TaxonIndex in seq_along(query)){
    if (verbose == TRUE) {
      message(paste0("Now on query ",TaxonIndex, " of ", length(query), " ",query[TaxonIndex]))
    }
    
    local.result  <- catalog_search(query[TaxonIndex], type = "Species", unavailable, resolve, phrase)
    ptime_Start <- Sys.time()
    if (length(local.result) > 0) {
      
      current.query.dat <- data.frame(matrix(nrow = length(local.result),ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)#Set up data storage
      colnames(current.query.dat) <- tax_sum_columns
      
      current.query.dat$Query <- query[TaxonIndex]#Fill current query information
      for(result.index in 1:length(local.result)){
        #loop through results and parse data
        current.query.dat$Distribution[result.index] <- get_distribution(local.result = local.result[result.index])#parse distribution data 
        current.query.dat[result.index, 21:23] <- get_habitat(local.result = local.result[result.index])#parse and store habitat data
        families <- get_family(local.result = local.result[result.index])#obtain family/subfamily
        ifelse(length(families)<2, current.query.dat[result.index,18:19] <- c(families, NA),current.query.dat[result.index,18:19] <- families)#check and store family daa
        current.query.dat[result.index,7:9] <- get_current_status(local.result = local.result[result.index])#obtain status of taxon
        current.query.dat[result.index, 10:16] <- get_types(local.result = local.result[result.index])#obtain and store type information
        current.query.dat[result.index, c(2:4,6)] <- get_focal_name(local.result = local.result[result.index])#Obtain focal species, author, year, and description reference
        current.query.dat$TypeLocality[result.index]<-get_type_locality(local.result = local.result[result.index])#Obtian type locality
        current.query.dat[result.index, 24:25] <- get_iucn(local.result = local.result[result.index]) #Get IUCN date and status
        current.query.dat[result.index, 26] <- get_nomenclature_notes(local.result = local.result[result.index]) #Get nomenclature notes (tags in bold)
        current.query.dat[result.index, c(3, 27)] <- get_infrasub(local.author = current.query.dat[result.index, 3]) #Get infrasubspecific tags
        current.query.dat[result.index, 5] <- get_page_number(local.result = local.result[result.index], processed.input = current.query.dat[result.index, 3])
        current.query.dat[result.index, 28] <- get_miscellanea(local.result[result.index], processed.input = current.query.dat[result.index,])
        current.query.dat[result.index, 29] <- paste(TaxonIndex, result.index, sep = "_")
        wrongNAindex <- which(current.query.dat == "NA", arr.ind = T)
        current.query.dat[wrongNAindex] <- NA
      }
      all.tax.sum <- rbind.data.frame(all.tax.sum,current.query.dat)#store current data in data to return
      
      if (taxon.history == TRUE) {
        for (result.index in seq_along(local.result)) {
          tryCatch(expr = {
            ##### Set Up Data Storage Tables #####
            taxon.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Taxon history overall storage (will contain taxon.history.status and parsed.history.dat)
            taxon.history.status <- data.frame(matrix(nrow = 2, ncol = length(tax_hist_columns))) # Nominal and current history storage
            parsed.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Parsed taxon history by catalog entry
            colnames(taxon.history.dat) <- colnames(taxon.history.status) <- colnames(parsed.history.dat) <- tax_hist_columns
            
            ##### Process Nominal and Current Taxon History #####
            taxon.history.status$Query <- current.query.dat$Query[result.index] # Add query info
            taxon.history.status$NominalTaxa <-current.query.dat$NominalTaxa[result.index] # Add focal name
            taxon.history.status$ResultCode <- current.query.dat$ResultCode[result.index]
            if (is.na(current.query.dat$Status[result.index])) {
              taxon.history.status$Status <- c("Nominal Species", "Currently no status")
            } else if (current.query.dat$Status[result.index] == "Uncertain") {
              taxon.history.status$Status <- c("Nominal Species", "Currently uncertain")
            } else if (current.query.dat$Status[result.index] == "Unknown") {
              taxon.history.status$Status <- c("Nominal Species", "Currently unknown")
            } else if (current.query.dat$Status[result.index] == "Synonym" && is.na(current.query.dat$CurrentNomenclature[result.index])) {
              taxon.history.status$Status <- c("Nominal Species", "Currently synonym")
            } else {
              taxon.history.status$Status <- c("Nominal Species","Current") # Add status types
            }
            taxon.history.status$RecognizedNomenclature <- c(current.query.dat$NominalTaxa[result.index],current.query.dat$CurrentNomenclature[result.index]) # Add in recognized nomenclature
            taxon.history.status$Authority <- c(current.query.dat$Author[result.index],current.query.dat$CurrentAuthority[result.index]) # Add authority info
            taxon.history.status$RefNo <- c(current.query.dat$DescriptionRef[result.index], NA) # Add in description ref 
            
            ##### Process Taxon History #####
            history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022"), c(". <b>"," <b>")) # Extract bulleted information from catalog
            if(is.na(history)) {
              history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022","\\u2022"), c("[A-z]+idae",". Distribution",". Habitat"), fixed = FALSE) # If history is not found due to lack of status, base off of family, distribution, or habitat
            }
            changes <- unlist(stringr::str_split(history, "\\u2022")) # Split history records on bullets
            changes <- gsub("et al,","et al.",changes) # Handle issues of commas in et al.
            
            for (change.index in seq_along(changes)) { # Process each bullet one by one
              tax.status <- qdapRegex::ex_between(changes[change.index],left = c("",""),right = c("<i>"," --"))[[1]][1] # Obtain taxa status for current bullet
              if (grepl("<a", tax.status, fixed = T)) {tax.status <- str_remove_all(tax.status, "\\<a.*?\\>|\\<\\/a\\>")}
              nom.status <- qdapRegex::ex_between(changes[change.index],left = "<i>",right = "</i>") # Obtain the nomenclature status
              if (grepl("[Hh]ybrid", tax.status)) {
                nom.status <- nom.status[[1]][1:2]
                if (is.na(nom.status[2])) {
                  nom.status <- nom.status[1]
                }
              } else {
                nom.status <- nom.status[[1]][1]
              }
              # Find any notes before authorship information for current bullet
              notes <- gsub(pattern = "^[12]\\d\\d\\d\\)?\\,?| --|<i>|</i>", replacement = "",
                            x = stringr::str_extract(string = changes[change.index], pattern = "[12]\\d\\d\\d\\)?.* --"))
              if (grepl("[Hh]ybrid", tax.status)) {
                if (grepl(paste0("^ and ", nom.status[2], " \\(.*?\\)$"), notes)) {
                  notes <- NA
                } else if (grepl(paste0("^ x ", nom.status[2], " \\(.*?\\)$"), notes)) {
                  notes <- NA
                } 
              }
              references <- tax_hist_changes(changes[change.index])
              nom.refs <- references[[1]]
              nom.refs.no <- references[[2]]
              
              current.taxon.history <- data.frame(matrix(nrow = ifelse(length(nom.refs) > 0, length(nom.refs), 0),ncol = length(tax_hist_columns)),stringsAsFactors = FALSE)#for the current history store data
              colnames(current.taxon.history) <- tax_hist_columns
              
              current.taxon.history$Query <-query[TaxonIndex]#fill with query info
              current.taxon.history$NominalTaxa <- current.query.dat$NominalTaxa[result.index]#fill in focal name
              current.taxon.history$ResultCode <- current.query.dat$ResultCode[result.index]
              current.taxon.history$Status <- tax.status#add status of taxon
              current.taxon.history$Notes <- ifelse(notes == "",NA, notes)#add in notes on status
              current.taxon.history$RecognizedNomenclature <- ifelse(length(nom.status)==2, paste(nom.status[1], "x", nom.status[2]), nom.status)#add in recognized name
              current.taxon.history$Authority <- ifelse(nchar(nom.refs) > 0, nom.refs, NA)#add in authority info
              current.taxon.history$RefNo <- nom.refs.no#put in reference number
              parsed.history.dat<-rbind(parsed.history.dat,current.taxon.history)#combine with other taxon history data
            }
            taxon.history.dat<-rbind(taxon.history.status[1,],parsed.history.dat,taxon.history.status[2,])#combine current taxon history with current/nominal data
            if(taxon.history == T) {all.taxon.history <- rbind.data.frame(all.taxon.history, taxon.history.dat)}#combine current history with all history to be returned
          },
          error = function(cond) {
            message(paste("Error when parsing taxon history for", current.query.dat$NominalTaxa[result.index]))
          })
        }
        del <- which(is.na(all.taxon.history$Status))
        RowsToDelete <- c()
        for (i in seq_along(del)) {if (is.na(all.taxon.history[del[i], 4]) && is.na(all.taxon.history[del[i], 5]) && is.na(all.taxon.history[del[i], 6]) && is.na(all.taxon.history[del[i], 7])) {RowsToDelete <- c(RowsToDelete, del[i])}}
        if(length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
      }
    } else if (length(local.result) == 0) {
      warning(paste0("No results found for supplied taxon ",query[TaxonIndex]))#Warn users if query is not found
    }
    ptime_End <- Sys.time()
    ptime_Total <- ptime_End - ptime_Start
    sleep.time <- ifelse(ptime_Total < 10, 10-ptime_Total, 0) #Adjust sleep time between server calls based processing time
    if (TaxonIndex != length(query)) {Sys.sleep(sleep.time)} #System sleep between calls
  }
  
  if (taxon.history == TRUE) {
    all.taxon.history.cleaned <- lapply(all.taxon.history, function(x) gsub("amp;", "", x))#kill &
    all.taxon.history.cleaned <- as.data.frame(all.taxon.history.cleaned)#make data frame
    if (common.name == TRUE) {
      all.results <- list(all.tax.sum, all.taxon.history.cleaned, all.common.names)
      names(all.results) <- c("TaxonSummary", "TaxonHistory", "CommonNames")
      return(all.results)
    } else if (common.name == FALSE) {
      all.results <- list(all.tax.sum, all.taxon.history.cleaned)
      names(all.results) <- c("TaxonSummary", "TaxonHistory")
      return(all.results)
    }
  } else if (common.name == TRUE) {
    all.results <- list(all.tax.sum, all.common.names)
    names(all.results) <- c("TaxonSummary", "CommonNames")
    return(all.results)
  } else {
    return(all.tax.sum)
  }
  
}

####################################################################################
####################################################################################
####################################################################################
#
#  rcatfish_search_genus
#
####################################################################################
####################################################################################
####################################################################################

rcatfish_search_genus <- function(query, unavailable = FALSE, taxon.history = FALSE, resolve = FALSE, sleep.time = 10, phrase = FALSE, verbose = TRUE) {
  ##### Check Arguments for Parameters #####
  
  ##### Check that Windows is Using OpenSSL ##### 
  if((.Platform$OS.type == "windows") & (grepl(pattern = "\\(OpenSSL", curl::curl_version()$ssl_version) == TRUE)){
    stop("openSSL backend for curl is required for the Windows version of this package, but is not detected as being active. Please see the vignette on how to configure curl with openSSL for this function to work. You can access the vignette with the following: vignette('rcatfish')")
  }
  sleep.time <- ifelse(length(query)>1, sleep.time, 0) # Set sleep time for queries longer than 1
  
  #Make Tables to store results
  tax_sum_columns <- c("Query",	"NominalTaxa",	"Author",	"DescriptionRef", "DescriptionYear", "Status", "CurrentNomenclature", "CurrentAuthority",	"TypeSpecies", "Gender", "TypeBy",	"Family","Subfamily","Notes", "NomenclatureNotes", "AsSubgenus", "ResultCode")
  all.tax.sum <- data.frame(matrix(nrow = 0, ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)
  colnames(all.tax.sum) <- tax_sum_columns
  
  if (taxon.history == T) {
    tax_hist_columns <- c("Query","NominalTaxa","Status","RecognizedNomenclature","Authority","RefNo","Notes", "ResultCode")
    all.taxon.history <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns)))
    colnames(all.taxon.history) <- tax_hist_columns 
  }
  
  for (TaxonIndex in seq_along(query)) {
    if (verbose == TRUE) {
      message(paste0("Now on query ",TaxonIndex, " of ", length(query), " ",query[TaxonIndex]))
    }
    
    local.result  <- catalog_search(query[TaxonIndex], type = "Genus", unavailable, resolve, phrase)
    ptime_Start <- Sys.time()
    
    if (length(local.result) > 0) {
      current.query.dat <- data.frame(matrix(nrow = length(local.result),ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)#Set up data storage
      colnames(current.query.dat) <- tax_sum_columns
      
      current.query.dat$Query <- query[TaxonIndex]#Fill current query information
      for(result.index in 1:length(local.result)){
        #loop through results and parse data
        current.query.dat[result.index, c(2:5,16)] <- get_focal_genus(local.result = local.result[result.index])#add in the focal genus, author and year
        current.query.dat[result.index,6:8] <- get_current_status(local.result = local.result[result.index])#get genus status
        current.query.dat$TypeSpecies[result.index] <- get_type_species(local.result = local.result[result.index])#get type species of genus
        current.query.dat$Gender[result.index] <- get_gender(local.result = local.result[result.index])#get the gender of the genus
        current.query.dat$TypeBy[result.index] <- get_type_by(local.result = local.result[result.index])#get what the type is by
        families <- get_family_genus(local.result = local.result[result.index])#Get the family of the genus
        ifelse(length(families)<2, current.query.dat[result.index,12:13] <- c(families, NA),current.query.dat[result.index,12:13] <- families)#check for subfamilies and write to table
        current.query.dat$Notes[result.index] <- get_genus_notes(local.result = local.result[result.index])#Get notes on genus
        current.query.dat$NomenclatureNotes[result.index] <- get_nomenclature_notes(local.result = local.result[result.index]) #Add in nomenclature notes (bold tags in search results)
        current.query.dat$ResultCode[result.index] <- paste(TaxonIndex, result.index, sep = "_")
        wrongNAindex <- which(current.query.dat == "NA", arr.ind = T)
        current.query.dat[wrongNAindex] <- NA
      }
      all.tax.sum <- rbind.data.frame(all.tax.sum,current.query.dat)#store current data in data to return
      
      if (taxon.history == TRUE) {
        for (result.index in seq_along(local.result)) {
          tryCatch(expr = {
            ##### Set Up Data Storage Tables #####
            taxon.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Taxon history overall storage (will contain taxon.history.status and parsed.history.dat)
            taxon.history.status <- data.frame(matrix(nrow = 2, ncol = length(tax_hist_columns))) # Nominal and current history storage
            parsed.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Parsed taxon history by catalog entry
            colnames(taxon.history.dat) <- colnames(taxon.history.status) <- colnames(parsed.history.dat) <- tax_hist_columns
            
            ##### Process Nominal and Current Taxon History #####
            taxon.history.status$Query <- current.query.dat$Query[result.index] # Add query info
            taxon.history.status$NominalTaxa <-current.query.dat$NominalTaxa[result.index] # Add focal name
            taxon.history.status$ResultCode <- current.query.dat$ResultCode[result.index]
            taxon.history.status$Status <- c("Nominal Species","Current") # Add status types
            taxon.history.status$RecognizedNomenclature <- c(current.query.dat$NominalTaxa[result.index],current.query.dat$CurrentNomenclature[result.index]) # Add in recognized nomenclature
            taxon.history.status$Authority <- c(current.query.dat$Author[result.index],current.query.dat$CurrentAuthority[result.index]) # Add authority info
            taxon.history.status$RefNo <- c(current.query.dat$DescriptionRef[result.index], NA) # Add in description ref 
            
            ##### Process Taxon History #####
            history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022"), c(". <b>"," <b>")) # Extract bulleted information from catalog
            if(is.na(history)) {
              history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022","\\u2022"), c("[A-z]+idae",". Distribution",". Habitat"), fixed = FALSE) # If history is not found due to lack of status, base off of family, distribution, or habitat
            }
            changes <- unlist(stringr::str_split(history, "\\u2022")) # Split history records on bullets
            changes <- gsub("et al,","et al.",changes) # Handle issues of commas in et al.      
            
            for (change.index in seq_along(changes)) { # Process each bullet one by one
              tax.status <- qdapRegex::ex_between(changes[change.index],left = c("",""),right = c("<i>"," --"))[[1]][1] # Obtain taxa status for current bullet
              nom.status <- qdapRegex::ex_between(changes[change.index],left = "<i>",right = "</i>")[[1]][1] # Obtain the nomenclature status
              # Find any notes before authorship information for current bullet
              notes <- gsub(pattern = "^[12]\\d\\d\\d\\)?\\,?| --|<i>|</i>", replacement = "",
                            x = stringr::str_extract(string = changes[change.index], pattern = "[12]\\d\\d\\d\\)?.* --"))
              
              references <- tax_hist_changes(changes[change.index])
              nom.refs <- references[[1]]
              nom.refs.no <- references[[2]]
              
              current.taxon.history <- data.frame(matrix(nrow = ifelse(length(nom.refs) > 0, length(nom.refs), 0),ncol = length(tax_hist_columns)),stringsAsFactors = FALSE)#for the current history store data
              colnames(current.taxon.history) <- tax_hist_columns
              
              current.taxon.history$Query <-query[TaxonIndex]#fill with query info
              current.taxon.history$NominalTaxa <- current.query.dat$NominalTaxa[result.index]#fill in focal name
              current.taxon.history$ResultCode <- current.query.dat$ResultCode[result.index]
              current.taxon.history$Status <- tax.status#add status of taxon
              current.taxon.history$Notes <- ifelse(notes == "",NA, notes)#add in notes on status
              current.taxon.history$RecognizedNomenclature <- nom.status#add in recognized name
              current.taxon.history$Authority <- ifelse(nchar(nom.refs) > 0, nom.refs, NA)#add in authority info
              current.taxon.history$RefNo <- nom.refs.no#put in reference number
              parsed.history.dat<-rbind(parsed.history.dat,current.taxon.history)#combine with other taxon history data
            }
            taxon.history.dat<-rbind(taxon.history.status[1,],parsed.history.dat,taxon.history.status[2,])#combine current taxon history with current/nominal data
            if(taxon.history == T) {all.taxon.history <- rbind.data.frame(all.taxon.history, taxon.history.dat)}#combine current history with all history to be returned
            RowsToDelete <- which(is.na(all.taxon.history$TaxonHistory$Status) & is.na(all.taxon.history$TaxonHistory$RecognizedNomenclature) & is.na(all.taxon.history$TaxonHistory$Authority) & is.na(all.taxon.history$TaxonHistory$RefNo) & is.na(all.taxon.history$TaxonHistory$Notes))
            if (length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
          },
          error = function(cond) {
            message(paste("Error when parsing taxon history for", current.query.dat$NominalTaxa[result.index]))
          })
        }
        del <- which(is.na(all.taxon.history$Status))
        RowsToDelete <- c()
        for (i in seq_along(del)) {if (is.na(all.taxon.history[del[i], 4]) && is.na(all.taxon.history[del[i], 5]) && is.na(all.taxon.history[del[i], 6]) && is.na(all.taxon.history[del[i], 7])) {RowsToDelete <- c(RowsToDelete, del[i])}}
        if(length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
      }
    } else if (length(local.result) == 0) {
      warning(paste0("No results found for supplied taxon ",query[TaxonIndex]))#Warn users if query is not found
    }
    ptime_End <- Sys.time()
    ptime_Total <- ptime_End - ptime_Start
    sleep.time <- ifelse(ptime_Total < 10, 10-ptime_Total, 0)
    if (TaxonIndex != length(query)) {Sys.sleep(sleep.time)} #System sleep between calls
  }
  if (taxon.history == TRUE) {
    all.taxon.history.cleaned <- lapply(all.taxon.history, function(x) gsub("amp;", "", x))#kill &
    all.taxon.history.cleaned <- as.data.frame(all.taxon.history.cleaned)#make data frame
    all.results <- list(all.tax.sum, all.taxon.history.cleaned)
    names(all.results) <- c("TaxonSummary", "TaxonHistory")
    return(all.results)
  } else {
    return(all.tax.sum)
  }
}

#Test that rcatfish_search returns proper information


test_that("rcatfish_search returns proper information",{
  if((.Platform$OS.type == "windows") & (grepl(pattern = "\\(OpenSSL", curl::curl_version()$ssl_version) == TRUE)){
    expect_error(rcatfish_search(query = "Cichla", type = "Species"), regexp = cat("openSSL backend for curl is required for the Windows version of this package, but it not detected as being active. Please see the vignette on how to configure curl with openSSL for this function to work. You can access the vignette with the following: vignette('rcatfish')."))
  }else{
    query.hit <-try(rcatfish_search("valid as Abactochromis", "Species", taxon.history = TRUE, phrase = TRUE))
    if ("try-error"%in%class(query.hit)) {
      skip("could not connect to remote database")
    }else{
      expect_identical(as.numeric(query.hit$TaxonSummary$DescriptionYear), 1935)
      expect_identical(query.hit$TaxonHistory$Authority[1], "Trewavas [E.] 1935")
      expect_identical(query.hit$TaxonHistory$RefNo[1], "4451")
    }
  }
})

#' Retrieve the current version Number of the Eschmeyer's Catalog of Fishes Being Used
#' 
#' @description Returns the current version of Catalog of Fishes being used in the R session
#' @return Character Vector length of one with the Catalog of Fishes version
#' @examplesIf interactive()
#' \donttest{
#' #' my.version <- rcatfish_version()
#' }
#' @author Samuel R. Borstein
#' @references 
#' Fricke, R., Eschmeyer, W.N. & van der Laan, R. (Year Accessed). Eschmeyer's Catalog of Fishes: Genera, Species, References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp.
#' @export

rcatfish_version<-function(){
  homeCOF <- readLines('https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp',warn = FALSE)
  versionLine <- grep("version of",homeCOF)
  cof_version <- rvest::html_text(xml2::read_html(homeCOF[versionLine]))
  cof_version <- strsplit(cof_version," - |  ")[[1]][2]
  return(cof_version)
}

#Test the version of the catalog can be retrieved and parsed


test_that("#Test the version of the catalog can be retrieved and parsed",{
  GetVersion <-try(rcatfish_version())
    if ("try-error"%in%class(GetVersion)) {
      skip("could not connect to remote database")
      }else{
        expect_equal(grep(pattern = "Continuously updated since the early 1980s, around the turn of the month.", GetVersion), 1)
        if(lubridate::month(lubridate::now())== 1){#If January, as update is mid-month, check if the version is either the current year or past year 
          expect_in(as.numeric(stringr::str_extract(GetVersion, "\\d{4}")), c(lubridate::year(lubridate::now()),lubridate::year(lubridate::now())-1))
        }else{#else, check current year
          expect_equal(lubridate::year(lubridate::now()), as.numeric(stringr::str_extract(GetVersion, "\\d{4}")))
        }
    }
  }
)

#' Search the Catalog of Fishes in R
#'
#' @description Provides access to California Academy of Sciences Catalog of Fishes Database in R
#' @details 
#' Package: rcatfish
#' 
#' Type: Package
#' 
#' Title: Provides access to California Academy of Sciences Eschmeyer's Catalog of Fishes Database in R
#' 
#' Version: 1.0
#' 
#' 
#' License: GPL (>= 2)
#' 
#' This package allows user to query the California Academy of Sciences Eschmeyer's Catalog of Fishes through R.
#' Users can search for Genera, Species, and references, as well as query the number of species
#' described in each family or subfamily. A vignette going over the basic functions and
#' how to use them can be accessed with vignette("rcatfish-vignette").
#' 
#' @author Samuel Borstein, Brandon Dominy, Brian O'Meara.
#' Maintainer: Samuel Borstein <sam@borstein.com>
#' 
#' @seealso @seealso \code{\link{rcatfish_search}},\code{\link{rcatfish_references}},\code{\link{rcatfish_version}},\code{\link{rcatfish_collections}},\code{\link{rcatfish_classification}},\code{\link{rcatfish_glossary}},\code{\link{rcatfish_journals}},\code{\link{rcatfish_species_by}},\code{\link{rcatfish_updates}}
#' 
#' @name rcatfish
#"_PACKAGE"

#' Search for references in Eschmeyer's Catalog of Fishes
#' 
#' @description Search for references in the Catalog of Fishes by keyword or reference number.
#' @param query Either a character vector of keywords to search for or a numeric vector with reference numbers
#' @param type Either "RefNo" if searching for a reference number or "keyword" if searching by keywords
#' reference numbers.
#' @param sleep.time Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.
#' @param verbose Logical. Should query progress be messaged to the screen? Default is TRUE.
#' @details This function searches Catalog of Fishes references. Users can supply either a keyword 
#' (i.e. cichlidae, revision, etc.) or a reference number to retrieve reference information.
#' @return A data.frame containing columns for the query, reference number, and full citation.
#' @examplesIf interactive()
#' \donttest{
#' #Perform a search of references that contain the keyword Amphilophus.
#' my.refs<-rcatfish_references(query = "Amphilophus", type = "keyword")
#' #Perform a search of references based on a Catalog of Fishes reference number
#' my.refs<-rcatfish_references(query = 2787, type = "RefNo")
#' }
#' @author Samuel R. Borstein, Brian C. O'Meara
#' @references 
#' Fricke, R. (Year Accessed). Eschmeyer's Catalog of Fishes: References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp
#' @export
#' 
rcatfish_references<-function(query, type, sleep.time = 10, verbose = TRUE){
  if(length(type) > 1){#Check length of type. Must be 1.
    stop("Length of argument type must be 1")
  }
  
  stopifnot("Argument type must be either RefNo or keyword" = type %in% c("RefNo", "keyword"))#Error if type is not a valid option
  
  ResTable<-as.data.frame(matrix(ncol=3, nrow = 0))
  colnames(ResTable)<-c("Query","RefNo","Reference")
  raw.query <- query
  ifelse(type=="RefNo",query<-paste("\"ref. ",query,"\"",sep=""),query<-query)
  for(query.index in seq_along(query)){
    if (verbose == TRUE) {
      message(paste0("Now on query ",query.index, " of ", length(query), ", ",raw.query[query.index]))
    }
    result <- RCurl::postForm("https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp",
                              contains=query[query.index], tbl="Ref", submit="Submit",
                              style='POST',.encoding = "utf-8")#Perform search
    result.rv <- xml2::read_html(result)#parse results
    local.results <- as.character(rvest::html_elements(result.rv, '.result')[-1])#Strip out header
    #Strip out returns and form into list
    local.results <- local.results[!grepl('<p class=\"result\">\r\n</p>\n', local.results)]
    if(length(local.results)==0){
      warning(paste0("No results found for supplied query ",query[query.index]))
      next
    }
    Results2Parse<-StripFun(local.results)#Strip out html tags
    #Parse out the results
    for (refIndex in seq(1, length(Results2Parse),by = 2)){
      currentRef<-Results2Parse[c(refIndex,refIndex+1)]
      currentRef[1]<-gsub("\n\r\n", replacement = "",x = currentRef[1])#Kill returns
      SplitRefNo<-stringr::str_split_fixed(currentRef[1],"\\[",n = 2)#Sub out reference info
      RefNo <- stringr::str_extract_all(SplitRefNo[2], "[0-9]")#Grab the numbers
      RefNo<-as.numeric(paste0(RefNo[[1]], collapse = ""))#Paste them
      #Make reference from the two halves of info, authors and citation
      Reference<-trimws(paste0(gsub(") ", ").",SplitRefNo[1], fixed = T),gsub("\n","", currentRef[2])))
      current.res<-as.data.frame(matrix(nrow = 1,ncol = 3))
      colnames(current.res)<-colnames(ResTable)
      current.res[1,]<-c(query[query.index],RefNo,Reference)
      ResTable<-rbind(ResTable,current.res)
    }
    if (length(query)>1) {
      Sys.sleep(sleep.time)#Sleep between calls for non-overload if query is longer than 1
    }
  }
  return(ResTable)
}

#Test that rcatfish_references returns proper reference information


test_that("rcatfish_references returns proper information",{
  reference.hit <-try(rcatfish_references(query = "Heroina", type = "keyword"))
  if ("try-error"%in%class(reference.hit)) {
    skip("could not connect to remote database")
    }else{
      expect_identical(as.numeric(reference.hit$RefNo), 22574)
    }
  reference.hit2 <-try(rcatfish_references(query = "2541", type = "RefNo"))
  if ("try-error"%in%class(reference.hit2)) {
    skip("could not connect to remote database")
    }else{
      expect_identical(as.numeric(grep(pattern = "A review of the Sparidae and related families of perch-like fishes found in the waters of Japan", reference.hit2$Reference)), 1)
    }
  }
)

#' Accesses the Eschmeyer's Catalog of Fishes Guide to Fish Collections
#' 
#' @param query Character Vector of search terms to search for. Default is NULL.
#' @param abbreviation Character vector containing the musuem collection code(s). Default is NULL. 
#' @param country Character vector of country names to search for. Default is NULL.
#' @param phrase Logical. Should query be passed as a quoted phrase (e.g. "Museum of Zoology"). Default is FALSE.
#' @param sleep.time Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.
#' @param verbose Logical. Should query progress be messaged to the screen? Default is TRUE.
#' @return Data frame, in which each row is a natural history collection and each column the corresponding collection information (see itemized list below).
#' \itemize{
#'   \item Code - Character. Natural history collection abbreviation code.
#'   \item Name  - Character. Full name of the natural history collection.
#'   \item Country  - Character. Country of the natural history collection.
#'   \item OtherAbbr - Character. Other and previous abbreviation(s) used for that natural history collection.
#'   \item OldName - Character. Previous name(s) used by the natural history.
#'   \item NewName - Character. New name(s) of the natural history collection.
#'   \item WebPage - Character. URL for the webpage of the natural history collection if one exists.
#'   \item CollectionDB - Character. URL for accessing the collection database of the natural history collection if one exists.
#'   \item TypesDB - Character. URL for accessing the type database of the natural history collection if one exists.
#'   \item Remarks - Character. Additional information about the natural history collection (e.g. includes specimens from other collections, closed, etc.).
#'   \item Publications - Character. Comma separated character vector of Eschmeyer's Catalog of Fishes reference numbers citing the natural history collection.
#' }
#' @details This function retrieves and parses natural history collection data from the Eschmeyer's Catalog of
#' Fishes. Returned information includes the collection abbreviation, country, alternate abbreviations,
#' old and new collection names, websites and databases associated with the collection, remarks, and
#' publications associated with the collection.
#' 
#' It is important that search criteria match exactly how they are on the Catalog of Fishes, otherwise
#' data will not be found (case must match, spelling, etc.). Users wishing to query more than one query
#' mixed with either a country or abbreviation should note that all parameters must be of equal length, so if searching
#' two queries both in U.S.A., the country should be supplied twice (see the final example in the examples below).
#' @examplesIf interactive()
#' #Search Museum Code For UMMZ (University of Michigan Museum of Zoology)
#' my.collections <- rcatfish_collections(abbreviation = "UMMZ", country = NULL, 
#' query = NULL, verbose = TRUE)
#' \donttest{
#' #Search For Collections in France
#' my.collections <- rcatfish_collections(abbreviation = NULL, country = "France", query = NULL)
#' #Search For Collections in The United States that are in Illinois
#' my.collections <- rcatfish_collections(abbreviation = NULL, country = "U.S.A.", query = "Illinois")
#' #Search For Collections in the United States that are in California & Alaska.
#' my.collections <- rcatfish_collections(abbreviation = NULL, country = rep("U.S.A.",2), 
#' query = c("California","Alaska"), sleep.time = 10)
#' #Search for Collections with Museum of Zoology in the name
#' my.collections <- rcatfish_collections(query = "Museum of Zoology", phrase = TRUE)
#' }
#' @author Samuel R. Borstein
#' @references 
#' Fricke, R. & Eschmeyer, W.N. (Year Accessed). Eschmeyer's Catalog of Fishes: Guide to Fish Collections. https://researcharchive.calacademy.org/research/ichthyology/catalog/collections.asp.
#' @export

rcatfish_collections <- function(query = NULL, abbreviation = NULL, country = NULL, phrase = FALSE, sleep.time = 10, verbose = TRUE){
  collections.final<- data.frame(matrix(ncol = 11,nrow = 0))
  colnames(collections.final)<-c("Code","Name","Country","OtherAbbr","OldName","NewName","WebPage","CollectionDB","TypesDB","Remarks","Publications")
  max.length<-max(length(abbreviation), length(country), length(query))
  sleep.time <- ifelse(max.length>1, sleep.time, 0)#Set sleep time for queries longer than 1
  for(query.index in 1:max.length){
    if (verbose == TRUE) {
      message(paste0("Now on query ",query.index, " of ", max.length))
    }
    url <- ("https://researcharchive.calacademy.org/research/ichthyology/catalog/collections.asp")
    pgsession <- rvest::session(url)#start session
    pgform <- rvest::html_form(pgsession)#get form fields
    if(is.null(query)){
      vals <- rvest::html_form_set(pgform[[1]],Mus = abbreviation[query.index], Country = country [query.index], Mus_name = query[query.index])#Add in search values
    }else{
      vals <- rvest::html_form_set(pgform[[1]],Mus = abbreviation[query.index], Country = country [query.index], Mus_name = ifelse(phrase == TRUE, shQuote(query[query.index], type = "cmd"), query[query.index]))#Add in search values
    }
    res <- rvest::session_submit(pgsession, vals, submit = "xAction")#Submit search form
    result.rv <- xml2::read_html(res)#parse xml
    TableColl <- as.character(rvest::html_elements(result.rv, 'table')[-c(1:5)])#Get database lines
    local.result <- TableColl[!grepl('<p class=\"result\">\r\n</p>\n', TableColl)]#Kill spaces
    current.dat<- data.frame(matrix(ncol = 11,nrow = length(local.result)))
    colnames(current.dat)<-c("Code","Name","Country","OtherAbbr","OldName","NewName","WebPage","CollectionDB","TypesDB","Remarks","Publications")
    #local.result <- StripFun(local.result)
    #collection.code <- sapply(strsplit(local.result, "\\s+"), "[", 1)
    for(record.index in seq_along(local.result)){
      collection.dat <- qdapRegex::ex_between(local.result[record.index], left = "<b>",right = "</b></td>", fixed = TRUE)
      current.dat$Code[record.index] <- str_extract(string = collection.dat[[1]][1], "\\S{1,}")
      ifelse(grepl("See ",collection.dat[[1]][[2]]),current.dat$Name[record.index]<-paste0("See ",qdapRegex::ex_between(collection.dat[[1]][2],">","</a>")),current.dat$Name[record.index]<-collection.dat[[1]][[2]])
      #collection code is [[1]][1] and name is [[1]][2]
      #collection.name <- qdapRegex::ex_between(local.result, left = "\\n",right = "\\n", fixed = TRUE)[[1]][1]
      current.dat$Country[record.index] <- qdapRegex::ex_between(local.result[record.index], left = "Country:</b>",right = "</td>", fixed = TRUE)
      current.dat$OtherAbbr[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "Other Abbrev.:</b>", right = ".")
      current.dat$OldName[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "Old Name:</b>", right = "<b>")   
      current.dat$NewName[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "New Name:</b>", right = "<b>", fixed = TRUE)
      current.dat$WebPage[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "<b>Main site:</b> <a href=\"", right = "\\\"")
      current.dat$CollectionDB[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "<b>Coll. database</b>: <a href=\"", right = "\\\"")
      current.dat$TypesDB[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "<b>Types database</b>: <a href=\"", right = "\\\"")
      current.dat$Remarks[record.index] <- qdapRegex::ex_between(local.result[record.index],left = "Remarks:</b>", right = "</td>")
      current.dat$Publications[record.index] <-paste(qdapRegex::ex_between(local.result[record.index],left = "getref.asp?id=", right = "\"", fixed = TRUE)[[1]],sep = ",",collapse = ",")
    }
    collections.final<-rbind.data.frame(collections.final,current.dat)
    Sys.sleep(sleep.time)
  }
    return(collections.final)
}

#Test that rcatfish_collections returns proper collection information.


test_that("Test that rcatfish_collections returns proper collection information",{
  GetCollection <-try(rcatfish_collections(query = NULL, abbreviation = "UMMZ", country = NULL))
  if ("try-error"%in%class(GetCollection)) {
    skip("could not connect to remote database")
  }else{
    expect_equal(unlist(GetCollection$Name), "University of Michigan Museum of Zoology, Ann Arbor, Michigan, U.S.A.")
    expect_equal(unlist(GetCollection$Country), "U.S.A.")
  }
})

#' Access the Eschmeyer's Catalog of Fishes Glossary
#' 
#' @description This function is used to access the Eschmeyer's Catalog of Fishes Glossary.
#' 
#' @details This function displays the Glossary tab of Eschmeyer's Catalog of Fishes. The glossary contains definitions for various terms that are important to the discussion of taxonomy and are mentioned throughout the database.
#' @return A dataframe with three columns. Some terms have sub-terms (e.g. synonym will have definitions for junior, objective, and subjective in addition to defining synonym itself). The first column represents the name of the main term, the second column represents the name of sub-terms, and the final column provides the definition for these terms. 
#' @examplesIf interactive()
#' \donttest{
#' # Obtain the Eschmeyer's Catalog of Fishes Glossary
#' myGlossary <- rcatfish_glossary()
#' }
#' @author Samuel R. Borstein
#' @references 
#' van der Laan, R., Fricke, R. & Fong, J. (Year Accessed). Eschmeyer's Catalog of Fishes: Glossary. https://www.calacademy.org/scientists/catalog-of-fishes-glossary/.
#' @export

rcatfish_glossary <- function(){
  url <- "https://www.calacademy.org/scientists/catalog-of-fishes-glossary/"
  page.html <- xml2::read_html(url) # Read in html from lin
  PageFields <- as.character(rvest::html_elements(page.html, '.field-items')[3])
  PageFieldsCleaned <- gsub("<div class=\"field-items\"><div class=\"field-item even\">\n<p>", "",PageFields)
  PageFieldsCleaned2 <- gsub("\n</ul>\n</div></div>", "",PageFieldsCleaned)
  Definitions <- str_split(PageFieldsCleaned2, "\n<p>")[[1]]
  Definitions <- str_replace(Definitions, "&amp;", "&")
  GlossaryData <- as.data.frame(matrix(nrow = 0, ncol = 3))
  colnames(GlossaryData) <- c("Term","SubTerm","Definition")
  for(i in 1:length(Definitions)){
    CurrentTerm <- qdapRegex::ex_between(left = "^<strong>",right = "</strong>",text.var = Definitions[i], fixed = F)
    if(length(grep(pattern = "li>\n<", Definitions[i])) > 0){
      SubTerms <- str_split(Definitions[i],pattern = "<li>\n<")
      SubTermsCleaned <- SubTerms[[1]][-1]
      CurrentData <- as.data.frame(matrix(nrow = length(unlist(SubTerms)), ncol = 3))
      colnames(CurrentData) <- c("Term","SubTerm","Definition")
      CurrentData$Term <- as.vector(CurrentTerm)
      SubTermName <- qdapRegex::ex_between(text.var = SubTermsCleaned, left = "strong>", right = "</strong>.")
      CurrentData$SubTerm[2:length(unlist(SubTerms))] <- as.vector(SubTermName)
      CurrentData$Definition[1] <- qdapRegex::ex_between(text.var = SubTerms[[1]][1], left = "</strong>. ", right = "</p>")[[1]][1]
      CurrentData$Definition[2:length(unlist(SubTerms))] <- unlist(qdapRegex::ex_between(text.var = SubTermsCleaned, left = "</strong>. ", right = "</li>"))
      GlossaryData <- rbind(GlossaryData, CurrentData)
    }else{
      CurrentData <- as.data.frame(matrix(nrow = 1, ncol = 3))
      colnames(CurrentData) <- c("Term","SubTerm","Definition")
      CurrentData$Term <- CurrentTerm
      CurrentData$Definition <- qdapRegex::ex_between(text.var = Definitions[i], left = "</strong>. |</strong> ", right = "</p>")[[1]][1]
      CurrentData$Definition <- gsub("^. ","", qdapRegex::ex_between(text.var = Definitions[i], left = "</strong>", right = "</p>")[[1]][1])
      GlossaryData <- rbind(GlossaryData, CurrentData)
    }
  }
  return(GlossaryData)
}

#Test that rcatfish_glossary returns proper definitions


test_that("Test that rcatfish_glossary returns proper definitions",{
  glossary.hit <-try(rcatfish_glossary())
  if ("try-error"%in%class(glossary.hit)) {
    skip("could not connect to remote database")
    }else{
      expect_equal(glossary.hit$Definition[3], "An Article of the Code of Zoological Nomenclature.")
    }
  }
)

#' Strips out html tags
#' 
#' @param htmlString character vector of html strings.
#' @details Internal function to strip out html tags into pure text for later processing with exported functions of rcatfish
#' @return Character vector 
#' @author Samuel R. Borstein
#' @noRd

StripFun <- function(htmlString){
  read.html <- lapply(htmlString, xml2::read_html)
  stripped <- lapply(read.html, rvest::html_text)
  StrippedResults <- unlist(stripped)
  return(StrippedResults)
}

#' Access the Eschmeyer's Catalog of Fishes Classification
#' 
#' @description This function is used to access the Eschmeyer's Catalog of Fishes classification of fishes
#' 
#' @details This function displays the classification tab of Eschmeyer's Catalog of Fishes. The data are returned in a tabular framework that progresses from most to least inclusive from left to right. The function will return the class, order, suborder, family, subfamily, author, and common name of the family or subfamily.
#' @return A dataframe with seven columns. From left to right (and most inclusive to least inclusive), the columns represent class, order, suborder, family, subfamily, author, and the common name of the family or subfamily.
#' @examplesIf interactive()
#' #Obtain the Eschmeyer's Catalog of Fishes Classification
#' \donttest{
#' myClassification <- rcatfish_classification()
#' }
#' @author Samuel R. Borstein
#' @references 
#' van der Laan, R., Fricke, R. & Eschmeyer, W.N. (Year Accessed). Eschmeyer's Catalog of Fishes: Classification. https://www.calacademy.org/scientists/catalog-of-fishes-classification/.
#' @export

rcatfish_classification <- function() {
  url <- "https://www.calacademy.org/scientists/catalog-of-fishes-classification/"
  pagedat <- rvest::read_html(url,encoding = "utf-8") # Read html from page, create xml document object
  lists <- rvest::html_elements(pagedat, 'ul') # Obtain all lists, creates nodesets
  text <- rvest::html_text(lists) # Convert lists to text
  
  index <- which(grepl("^Class", text))
  
  Classes <- c()
  for (i in seq_along(index)) {
    start <- index[i]
    end <- index[i+1]-1
    if (is.na(end)) {end <- length(text)}
    output <- paste(text[start:end], collapse = "\\\n")
    Classes <- c(Classes, output)
  }
  
  splitClasses <- unique(unlist(str_split(Classes, "\\\n")))
  taxonomicIndices <- which(grepl("^Class |^Order |^Suborder |^Family |^Subfamily ", splitClasses))
  entries <- splitClasses[taxonomicIndices]
  
  results <- data.frame(matrix(nrow = 0, ncol = 7)) #Create dataframe to store data
  colnames(results) <- c("Class", "Order", "Suborder", "Family", "Subfamily", "Author","CommonName")
  
  class <- order <- subord <- family <- subfam <- author <- comname <- NA
  
  for (i in seq_along(entries)) {
    current.results <- data.frame(matrix(nrow = 1, ncol = 7))
    colnames(current.results) <- c("Class", "Order", "Suborder", "Family", "Subfamily", "Author","CommonName")
    
    if (grepl(pattern = "^Class", entries[i])) {
      class <- substr(entries[i], 7, nchar(entries[i]))
      order <- suborder <- family <- subfamily <- author <- commonname <- NA
    } else if (grepl(pattern = "^Order", entries[i])) {
      order <- substr(entries[i], 7, nchar(entries[i]))
      suborder <- family <- subfamily <- author <- commonname <- NA
    } else if (grepl(pattern = "^Suborder", entries[i])){
      suborder <- substr(entries[i], 10, nchar(entries[i]))
      family <- subfamily <- author <- commonname <- NA
    } else if (grepl(pattern = "^Family", entries[i])) {
      family <- substr(entries[i], 8, unlist(gregexpr("idae", entries[i]))+4)
      subfamily <- NA
      author <- unlist(qdapRegex::ex_between(left = "idae ", right = "(", text.var = entries[i]))
      commonname <- gsub("\\(","", x = str_extract(string = entries[i], pattern = "(?=\\([:alpha:]).+?(?=\\))"))
    } else if (grepl(pattern = "^Subfamily", entries[i])) {
      subfamily <- substr(entries[i], 11, unlist(gregexpr("inae", entries[i]))+4)
      author <- unlist(qdapRegex::ex_between(left = "inae ", right = "(", text.var = entries[i]))
      commonname <- gsub("\\(","", x = str_extract(string = entries[i], pattern = "(?=\\([:alpha:]).+?(?=\\))"))
    }
    
    current.results$Class <- trimws(class)
    current.results$Order <- trimws(order)
    current.results$Suborder <- trimws(suborder)
    current.results$Family <- trimws(family)
    current.results$Subfamily <- trimws(subfamily)
    current.results$Author <- trimws(author)
    current.results$CommonName <- trimws(commonname)
    
    results <- rbind.data.frame(results, current.results)
  }
  
  return(results)
}

#Test that rcatfish_classification returns proper information


test_that("Test that rcatfish_classification returns proper groupings",{
  classification.hit <-try(rcatfish_classification())
  if ("try-error"%in%class(classification.hit)) {
    skip("could not connect to remote database")
    }else{
      expect_equal(classification.hit$CommonName[which(classification.hit$Family == "Lamnidae")], "mackerel sharks")
      expect_equal(classification.hit$Author[which(classification.hit$Family == "Lamnidae")], "Bonaparte 1835")
      expect_equal(classification.hit$Class[which(classification.hit$Family == "Lamnidae")], "Elasmobranchii")
    }
  }
)

---
title: "rcatfish Tutorial"
author: "Samuel R. Borstein, Brandon E. Dominy"
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
  html_document:
    keep_md: true
vignette: >
  %\VignetteIndexEntry{rcatfish Tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
  
# 1: Introduction
  
This is a tutorial for using the R package `rcatfish`. `rcatfish` provides access to the California Academy of Sciences Eschmeyer's Catalog of Fishes within R (Eschmeyer et al., 1998, Fricke et al., 2025, https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp). The Catalog of Fishes database is the gold standard for fish taxonomy as it provides thorough citations of the taxonomic history of fishes and is updated continuosly via standard monthly releases. While there are other packages within R that can be used for checking the taxonomy of organisms, including fishes (i.e. `rfishbase`, `taxize`, `ritis`, etc.), the databases accessed by these packages lacks the expansive information on the taxonomic history of fishes and are typically not up to date on the cutting edge of fish systematics.`rcatfish` introduces functions to access the various information in the California Academy of Sciences Eschmeyer's Catalog of Fishes. This tutorial provides a basic introduction into using the package and its functions.

Please note when using this package that it is intended to act solely as an interface to Eschmeyer's Catalog of Fishes and is not affiliated with California Academy of Sciences. As Eschmeyer's Catalog of Fishes is only published in a non-machine-readable format, `rcatfish` is intended to parse the catalog data into a format more suitable for analysis through web scraping. As such, `rcatfish` will only be as accurate as the data in the published catalog, and spelling or formatting errors may arise due to inconsistencies in Eschmeyer's Catalog of Fishes data entry.

# 2: Installation
## 2.1: Installation From CRAN
In order to install the stable CRAN version of the `rcatfish` package:

```
install.packages("rcatfish")
```
## 2.2: Installation of Development Version From GitHub
While we recommend use of the stable CRAN version of this package, we recommend using the package `devtools` to temporarily install the development version of the package from GitHub if for any reason you wish to use it:
  
```
#1. Install 'devtools' if you do not already have it installed:
install.packages("devtools")

#2. Load the 'devtools' package and temporarily install the development version of
#'dietr' from GitHub:
library(devtools)
dev_mode(on=T)
install_github("sborstein/rcatfish")  # install the package from GitHub
library(rcatfish)# load the package

#3. Leave developers mode after using the development version of 'rcatfish' so  it will not remain on your systempermanently.
dev_mode(on=F)
```

## 2.3 Installation Requirements for Windows OS users

To make https connections, the dependencies of rcatish utilize curl. In versions of Windows 7 and up, the curl implementation in R can use either `openSSL` or Windows Secure Channel (`SChannel`). Only one of these options can be active at a time and the default is `Schannel`, which conflicts with this package. To see which one you have active, you can do the following:
  
```
curl::curl_version()$ssl_version
```

In the output you may have more than one option. The ones in parentheses are not in use, while the ones lacking parentheses are in use. If you have `Schannel` in use, you will need to add the following line to your `~/.Renviron` file to have curl use openSSL.

```
CURL_SSL_BACKEND=openssl
```

This can be added manually or can be done directly in R by running the following line of code.

```
write('CURL_SSL_BACKEND=openssl', file = "~/.Renviron", append = TRUE)
```

After adding this line to your `~/.Renviron`, re-start R. Then check the `curl_version` again. `Schannel` should now be in parentheses while your `openSSL` option should now lack parentheses, indicating it is in use.

```
curl::curl_version()$ssl_version
```
# 3: Using rcatfish
Once installed, you can load `rcatfish` and all of its functions/data:

```
library(rcatfish)
```
Upon loading, you will see a message showing you the version of the Catalog of Fishes as well as how to properly cite the Catalog of Fishes and this R package. In the event you need to get the version of the Catalog of Fishes at any point, you can use the function `rcatfish_version()` which has no arguments to return the date of the version of the Catalog of Fishes being accessed.

## 3.1: Basic data necessary to run rcatfish

To use the majority of functions in `rcatfish` you will need the following data as inputs. First, for all functions which search the catalog, a `query` is required. This is can be either a single search term or a vector of terms to be searched in series, the content of which will be dependent on the particular search function being used (e.g. when searching for references, `query` may be a catalog reference number).

Aside from a `query`, several functions require a `type` as well. This is used to differentiate the search method in functions that support multiple types (e.g. search by `type = genus` or `type = keyword`). Similar to `query`, the content of this parameter will vary based on the function it is being used in, but unlike `query` it is typically not able to be vectorized and has a specific set of options it must match in each function that calls it.

All other parameters used in this package are optional and unique to each individual function. Several of these will be discussed below, but each function's options can be seen by running `?function_name` or `help(function_name)`.

## 3.2: Using rcatfish to Search for Species and Genera in Eschmeyer's Catalog of Fishes (`rcatfish_search`)
  
To search the Catalog of Fishes's taxonomic records, the function `rcatfish_search` should be used. This function is equivalent to using the "Search Eschmeyer's Catalog" tab on the catalog's website. While it has several parameters with default arguments, only `query` and `type` must be specified. 

As an example, a search can be performed for all available species names in the family *Rhincodontidae* using the following function call: 
```
# Search CoF for Available Species Names in Rhincodontidae
rhinco_species <- rcatfish_search(query = "Rhincodontidae", type = "Species")
View(rhinco_species)
```
When viewing the created object `rhinco_species` above, you should see a large dataframe containing species results of that family.

### 3.2.1 The `type` Parameter in `rcatfish_search`
The `type` parameter allows you to specify what type of results you want returned and is the equivalent of changing the radio button on the "Search Eschmeyer's Catalog" tab between "Genera" and "Species" (the "References" option is available using the `rcatfish_references` function which will be discussed later). It has two acceptable inputs, either "Species" or "Genus". This parameter is not vectorizable, so any searches should either all be by genus or all be by species. There is no default option, so it must always be explicitly assigned a value during function calls. To see an example of the difference between the two options:
```
# Search Rhincodontidae by Species
by_species <- rcatfish_search(query = "Rhincodontidae", type = "Species")
View(by_species)

# Search Rhincodontidae by Genus
by_genus <- rcatfish_search(query = "Rhincodontidae", type = "Genus")
View(by_genus)
```
When viewing the outputs from above, you will notice that despite each search containing the same query, the results differ. Please always ensure you have the correct value assigned when searching the catalog.

### 3.2.2 The `query`, `phrase`,  `unavailable`, and `resolve` Parameters in `rcatfish_search`
The `query` parameter represents the keyword(s) to search for in the catalog. These can be any text found in the catalog's entries, including species names, genera, family and subfamily names, author names, type specimen information, and more. This argument is vectorizable, meaning multiple different queries can be passed in one function call. By default there will be a 10-second wait between each query, as the Catalog of Fishes requests at least this much time between requests. As an example:

```
# Search CoF for Using Multiple Queries
searchTerms <- c("Rhincodontidae", "Aldrichetta")
result <- rcatfish_search(query = searchTerms, type = "Species")
View(result)
```
As you can see with the created object `results` from the function call above, one dataframe is returned containing all of the data from both queries, with the first column showing which query returned each result.

By default, queries of more than one word will search for a separate instance of each word in the Catalog of Fish's entries. For example, `query = status uncertain` will search for any entries which contain the words "status" and "uncertain" anywhere in their text. If you wish to only search for entries which contain the exact phrase "status uncertain", you can specify this using the `phrase` parameter. By default this parameter is set to `FALSE`, but by explicitly setting  it to `TRUE` you can have `rcatfish_search` treat each independent query as an exact phrase. As an example, see the difference in results for the following two searches:

```
# Search Catalog of Fishes with phrase = FALSE
no_phrase <- rcatfish_search(query = "Mugil abu", type = "Species")
View(no_phrase)

# Search Catalog of Fishes with phrase = TRUE
yes_phrase <- rcatfish_search(query = "Mugil abu", type = "Species", phrase = TRUE)
View(yes_phrase)
```
This package also has some functionality for correcting misspellings in searches. By default this is not performed, however by setting `resolve = TRUE` within the `rcatfish_search` function call it can be toggled on. When this is set, `rcatfish` will perform the search as usual, however in the event that no results are found, it will attempt to resolve the search queries using fuzzy matching by finding the closest matches to them through the Global Names Verifier. See the difference below between these two options:
```
# Searching Catalog of Fishes with a Misspelled Species Name 

# Without Resolving Names
no_resolve <- rcatfish_search(query = "rhincodon tipus", type = "Species")
View(no_resolve)

# With Resolving Names
yes_resolve <- rcatfish_search(query = "rhincodon tipus", type = "Species", resolve = T)
View(yes_resolve)
```

The catalog also allows the searching of unavailable names. By default these names are excluded when running a search, but by changing the `unavailable` parameter to `TRUE` they can be included. See the difference below:
```
# Search Catalog of Fishes Without Unavailable Names
no_unavailable <- rcatfish_search(query = "Mitsukurinidae", type = "Species")
View(no_unavailable)

# Search Catalog of Fishes With Unavailable Names
yes_unavailable <- rcatfish_search(query = "Mitsukurinidae", type = "Species", unavailable = TRUE)
View(yes_unavailable)
```


### 3.2.3 The `common.name` Parameter in `rcatfish_search`
Currently, Eschmeyer's Catalog of Fishes does not include the common names of species. `rcatfish` does, however, have the capability of searching for species by common name by utilizing the `rfishbase` package. To do this, you can utilize the `common.name` parameter in `rcatfish_search`. Please not that searching by common names can only be performed on a species level, not by genus.

By default this parameter is set to `FALSE`. When explicitly changed to `TRUE`, the function will first match the `query` input of common names to any associated scientific names currently found in FishBase. Note that while it is still vectorizable in this format, you can not combine common names with other search terms (e.g. you can search for `query = c("Humphead Wrasse", "Channel Catfish")` but searching for `query = c("Humphead Wrasse", "Lophius piscatorius")` may return unexpected results). This parameter will return a list containing the normal `rcatfish_search` result dataframe and a second dataframe showing the common names provided and the taxonomic names that they were matched to. As an example:

```
# Search Catalog of Fishes by Common Name
common_name_result <- rcatfish_search(query = "Humphead wrasse", type = "Species", common.name = TRUE)
View(common_name_result) # The full list returned
View(common_name_result[[1]]) # The first dataframe in the list, the normal rcatfish_search output
View(common_name_result[[2]]) # The second dataframe in the list, the common names searched and their matches
```
You can search using common names from other languages if you so desire by setting the `language` parameter. By default it is set to "English".


### 3.2.4 The `taxon.history` Parameter in `rcatfish_search`
Each entry in Eschmeyer's Catalog of Fishes also contains a complete history of that entry's taxonomic status. By default, this is not captured with `rcatfish_search`, although it can be obtained by setting the `taxon.history` parameter to `TRUE`. When this is done, an additional dataframe is returned containing each result's original status, current status, and every change to its status made in between. This search can be performed both by species and by genus. Please note that, particularly for queries with a large number of changes in their history, searching with `taxon.history = TRUE` may take considerably longer than a typical search. As an example of what this may look like:

```
# Search Catalog of Fishes by Common Name
taxon_history_result <- rcatfish_search(query = "Platyrhina", type = "Genus", taxon.history = TRUE)

View(taxon_history_result) # The full list returned
View(taxon_history_result[[1]]) # The first dataframe in the list, the normal rcatfish_search output
View(taxon_history_result[[2]]) # The second dataframe in the list, the taxonomic histories of the results
```

### 3.2.5 Other Parameters in `rcatfish_search`
Several other parameters exist in the `rcatfish_search` function to modify minor aspects of the search function.

The `verbose` parameter toggles on and off the message displayed to the user when running a search (e.g. "Now on query 1 of 100"). By default it is set to `TRUE`. Messages can be disabled by changing it to `FALSE`.

The `sleep.time` query sets the length of time that the search function will wait between requests to the Catalog of Fishes's server when performing a search of multiple terms. This is set to 10 seconds as requested by the catalog. **This parameter should not be modified. Changing this value may result in blacklisting by the catalog.**

## 3.3: Using rcatfish to Search for References in Eschmeyer's Catalog of Fishes (`rcatfish_references`)

To search through references in Eschmeyer's Catalog of Fishes, the function `rcatfish_references` should be used. This function is equivalent to using the "Search Eschmeyer's Catalog" tab on the catalog's website and selecting the "References" radio button. It has the parameters `query` and `type`, both of which must be specified. 

The `query` parameter can search either by reference number in the catalog of by keyword and can be passed as either a single search term or a vector of terms. The type of search performed is dictated by the the `type` parameter, which will accept either "RefNo" to search by reference number or "keyword" to search by keyword. Note that when searching by reference number, the query can be passed either as an integer or as a character string (e.g. 41479 and "41479" will  return the same results). 

```
# Search references by keyword
keyword_reference <- rcatfish_references(query = "Tunisia", type = "keyword")

# Search references by reference number
RefNo_reference <- rcatfish_references(query = 41479, type = "RefNo")
```

`rcatfish_references` can be combined with a result from `rcatfish_search` to obtain all references associated with the resulting species. As an example:

```
# Search the catalog for a given species
search_result <- rcatfish_search(query = "Cichla cataractae", type = "Species")

# Retrieve references from resulting search
references <- rcatfish_references(query = search_result$DescriptionRef, type = "RefNo")
```

## 3.4: Using rcatfish to See Recent Catalog Updates

Eschmeyer's Catalog of Fishes receives monthly updates. These updates include changes to the taxonomic status of genera and species, changes related to authorship, and the addition of newly described taxa. Users cans see these updates by using the `rcatfish_updates` function. By default, this function takes no arguments, and will return all changes provided by the most recent update. However, users can specify if they want to return the catalog taxonomic changes, authorship changes, added genera, and added.species with simple `TRUE` or `FALSE`. For example, if we wanted to obtain all the changes in a version of the catalog, we can do either of the following:
  
```
updates <- rcatfish_updates()
```

or, we can set specific arguments to return specific update components. These are set to `TRUE` by default, but users can change these given their names.

```
updates <- rcatfish_updates(changes = TRUE, author.changes = TRUE, added.genera = TRUE, added.species = TRUE)

updates
```
We can see when running the above code that a list is returned of changes made in the newest edition of the catalog (which is updated once a month). This list will be of a variable length depending on which elements the user asked to return. Other elements of the returned list () are `Changes`, `AuthorshipChanges`, `AddedGenera`, and `AddedSpecies`, which contain the taxonomic changes, authorship changes, newly added genera to the catalog, and added species to the catalog respectively.
  
## 3.5: Using rcatfish to Obtain the Number of Species and Genera Described Per Taxonomic Entity

Eschmeyer's Catalog of Fishes provides information on the number of species and genera described per family and subfamily via a table on the following linked page (https://researcharchive.calacademy.org/research/ichthyology/catalog/SpeciesByFamily.asp). rcatfish provides access to this page as well as the ability to return species totals for higher taxonomic entities than just family and subfamily, such as orders and classes using the rcatfish_species_by function. This function simply takes a query that is a subfamily, family, class, or order that the user wishes to obtain data for. For example, if we want to return information for the family Cichlidae we can easily do so using the following:

```
rcatfish_species_by("Cichlidae")
```

We can see that this has returned a data frame containing the number of available and valid genera and species, as well as the number of genera and species described in the last decade for the family and all subfamilies in Cichlidae. However, while the Catalog of Fishes  does not report these figures at higher taxonomic levels, rcatfish can. We can obtain the number of described genera and species for the order Cichliformes, with the following.
  
```  
rcatfish_species_by("Cichliformes")
```

We can see that this has provided not just the number of genera and species in each family within the Cichliformes, but has also returned the total for the entire order, which is not reported on the Catalog of Fishes.


## 3.6: Browsing the Eschmeyer's Catalog of Fishes Classification
  
Eschmeyer's Catalog of Fishes provides a hierarchical classification of fishes organized by Class, Order, Suborder, Family, and Subfamily (https://www.calacademy.org/scientists/catalog-of-fishes-classification/). The `rcatfish` function `rcatfish_classification` provides access to this table. This function lacks arguments and can be simply called as followed.

```
# See Current Breakdown of Fish Classification, from Class Through Subfamily
fish_classification <- rcatfish_classification()

fish_classification
```
The function returns a data frame that progresses from left to right from most to least inclusive. In addition to providing the hierarchy for Class, Order, Suborder, Family, and Subfamily, the authorship of these taxonomic entities as well as their common name is returned.


## 3.7: Accessing Eschmeyer's Catalog of Fishes Glossary

  
```
# See a Glossary of Terms Used in the Catalog
glossary <- rcatfish_glossary()
```
We can see that the `glossary` object made in the line of code above creates a data frame containing technical terms used in the catalog along with their definitions and applicable sub-terms.

## 3.8: Using rcatfish to Obtain Information on the Journals Cited in the Catalog of Fishes

Besides just citations for references used in the The Catalog of Fishes, the Catalog of Fishes also provides various information on the journals used for references (https://researcharchive.calacademy.org/research/ichthyology/catalog/journals.asp). For example, the Catalog of Fishes provides information for ISSN numbers, publishers, and comments, such as name changes for journals. Information on the journals can be accessed in rcatfish using the rcatfish_journals function. This function simply takes the argument query which is a string to search for as well as if the argument phrase which is if the query should be passed in quotes while searching as a phrase. For example, to search for journals that are related to Texas, we can do the following:

```
rcatfish_journals("Texas")
```
We can see that most of these contain Texas in the title, or information on how one, "Contributions in Marine Science" is a continuation of Publications of the Institute of Marine Science, University of Texas.

Note that passing the query as a phrase may impact the success of the search. For example, if we wanted to search for Journal of Zoology, the search will fail if we do not pass the query as a phrase as it will look for each word separately.

```
rcatfish_journals("Journal of Zoology")
```

We can successfully search for this query by invoking phrase = TRUE in the arguments:

```
rcatfish_journals("Journal of Zoology", phrase = TRUE)
```

## 3.9: Using rcatfish to Search the Eschmeyer's Catalog of Fishes Guide to Fish Collection

Eschmeyer's Catalog of Fishes provides information, such as collection abbreviations, locality, previous names, and online access for museum collections with fish holdings (https://researcharchive.calacademy.org/research/ichthyology/catalog/collections.asp). This information can be accessed through rcatfish via the rcatfish_collections function. rcatfish_collections allows users to search for collections by abbreviation, country, or query term. For example, if we knew the museum abbreviation we wanted to search for, such as the UMMZ for the University of Michigan Museum of Zoology, we could do the following by simply providing UMMZ to the abbreviation argument:

```
rcatfish_collections(abbreviation = "UMMZ", country = NULL, query = NULL, verbose = TRUE)
```

We can also pass information to more than one field. This can be useful for narrowing down collection results, such as for countries that have a lot of natural history collections. For this example, lets search for collections in the United States of America and query for collections in California and Alaska. Note that to do queries longer than 1, we must ensure that all arguments are the same length. So, in this case, we need to pass the country twice in our search as follows:

```
rcatfish_collections(abbreviation = NULL, country = rep("U.S.A.",2), query = c("California","Alaska"), sleep.time = 10)
```

We may also want to query a phrase, such as "Museum of Zoology" to get a list of collections that contain that name across all collections (similar to what was covered for `rcatfish_journals`). In order to do more complex queries that are phrases, we need to use the `phrase = TRUE` argument. We can do the following search as such:

```
rcatfish_collections(query = "Museum of Zoology", phrase = TRUE)
```

## 4.0: Troubleshooting

Most of the functions in `rcatfish` require a stable internet connection to run as it connects to the online Catalog of Fishes database. If you run into problems, we recommend checking your internet connection as well as visiting the California Academy of Sciences Eschmeyer's Catalog of Fishes site (https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp), to ensure that it is not down for routine maintenance.

Should you find any entries which appear to return different data than anticipated, check the Catalog of Fishes directly  to confirm the error and then use the issues section on Github (repo sborstein/rcatfish). Remember that `rcatfish` is designed to return exactly what is published and will capture any mistakes that are directly in the catalog.

Please note that the authors of this package are not affiliated with Eschmeyer's Catalog of Fishes nor the California Academy of Sciences. As such, we are not able to correct any errors that exist on the Catalog of Fishes or fix/troubleshoot any issues with the Catalog of Fishes itself.

## 5.0: Final Comments
Further information on the functions and their usage can be found in the help files `help(package=rcatfish)`.

For any further issues and questions send an email with subject 'rcatfish support' to borstein@txstate.edu or post to the issues section on GitHub.

## References:

Eschmeyer WN (1998). Catalog of Fishes California Academy of Sciences, San Francisco, California, 2905 pp.

Fricke R (2026). Eschmeyer's Catalog of Fishes: References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp.

Fricke R, Eschmeyer WN (2026). Eschmeyer's Catalog of Fishes: Guide to Fish Collections. https://researcharchive.calacademy.org/research/ichthyology/catalog/collections.asp.

Fricke R, Eschmeyer WN (2026). Eschmeyer’s Catalog of Fishes: Journals. https://researcharchive.calacademy.org/research/ichthyology/catalog/journals.asp.

Fricke R, Eschmeyer WN, Fong JD (2026). Eschmeyer’s Catalog of Fishes: Species by family/subfamily in the Catalog of Fishes. https://researcharchive.calacademy.org/research/ichthyology/catalog/SpeciesByFamily.asp.


Fricke R, Eschmeyer WN, van der Laan R (2026). Eschmeyer's Catalog of Fishes: Genera, Species, References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp.

Fricke R, van der Laan R, Fong JD (2026). Eschmeyer’s Catalog of Fishes: Changes and Additions. https://researcharchive.calacademy.org/research/ichthyology/catalog/ChangeSummary.asp.

van der Laan R, Fricke R, Eschmeyer WN (2026). Eschmeyer's Catalog of Fishes: Classification. https://www.calacademy.org/scientists/catalog-of-fishes-classification/.

van der Laan R, Fricke R, Fong J (2026). Eschmeyer's Catalog of Fishes: Glossary. https://www.calacademy.org/scientists/catalog-of-fishes-glossary/.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_version.R
\name{rcatfish_version}
\alias{rcatfish_version}
\title{Retrieve the current version Number of the Eschmeyer's Catalog of Fishes Being Used}
\usage{
rcatfish_version()
}
\value{
Character Vector length of one with the Catalog of Fishes version
}
\description{
Returns the current version of Catalog of Fishes being used in the R session
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
\donttest{
#' my.version <- rcatfish_version()
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R., Eschmeyer, W.N. & van der Laan, R. (Year Accessed). Eschmeyer's Catalog of Fishes: Genera, Species, References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp.
}
\author{
Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_journals.R
\name{rcatfish_journals}
\alias{rcatfish_journals}
\title{Search for journals in the Catalog of Fishes}
\usage{
rcatfish_journals(query, phrase = FALSE, sleep.time = 10, verbose = TRUE)
}
\arguments{
\item{query}{Character vector to search for.}

\item{phrase}{Logical. Should query be passed as a quoted phrase (e.g. "Journal of Zoology"). Default is FALSE.}

\item{sleep.time}{Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.}

\item{verbose}{Logical. Should query progress be messaged to the screen? Default is TRUE.}
}
\value{
Two column data frame. The first column contains the name of the query and the second column contains information on the journal.
}
\description{
Search for journals in the Catalog of Fishes
}
\details{
This function returns information on the journals cited in the Eschmeyer's Catalog of Fishes.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
\donttest{
my.journals<-rcatfish_journals(query="Cichlid")
my.journals<-rcatfish_journals(query="Evolution")
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R. & Eschmeyer, W.N. (Year Accessed). Eschmeyer’s Catalog of Fishes: Journals. https://researcharchive.calacademy.org/research/ichthyology/catalog/journals.asp
}
\author{
Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_species_by.R
\name{rcatfish_species_by}
\alias{rcatfish_species_by}
\title{Find number of available fish names, number of valid species, and number of descriptions in the last 10 years by family or subfamily}
\usage{
rcatfish_species_by(query, verbose = TRUE)
}
\arguments{
\item{query}{A character vector of taxa to search for. For users wishing to obtain the information for all fishes please query "Totals".}

\item{verbose}{Logical. Should query progress be messaged to the screen? Default is TRUE.}
}
\value{
A list with each element containing a data frame for each query. List elements are named by the queried taxa.
The list contains information on the number of genera and species available, valid, and described in the last 10 years. An itemized list describing the contents in the columns of the data frame(s) returned is described below.
#' \itemize{
\item Class - Character. Taxonomic class.
\item Order - Character. Taxonomic order.
\item Family - Character. Taxonomic family
\item Subfamily - Character. Taxonomic subfamily.
\item Available.Genera - Integer. Number of available genera for the taxonomic rank.
\item Valid.Genera - Integer. Number of valid genera for the taxonomic rank.
\item Genera.Last.Ten.Years - Integer. Number of genera described over the past ten years for the taxonomic rank.
\item Available.Species - Integer. Number of available species for the taxonomic rank.
\item Valid.Species - Integer. Number of valid species for the taxonomic rank.
\item Species.Last.Ten.Years - Integer. Number of species described over the past ten years for the taxonomic rank.
}
}
\description{
This function retrieves and parses data on the number of genera and species per family from the
Eschmeyer's Catalog of Fishes. It returns a table containing the number of genera and species that are available, valid,
and described in the last 10 years in its columns. Each row represents either a class, order, family, or subfamily, so
some columns for higher taxonomic groups may have NA for lower taxonomic groups (e.g. if you Search for Cypriniformes, the row
containing Cypriniformes will have NA for family and subfamily as it is above those taxonomic levels).

Given the function reads in the page containing the table, for speed, it is highly recommended that if a user wishes to obtain
information on numerous orders, families, or subfamilies that they pass them in as a single query. To do this, see the last example.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
\donttest{
#Search for the number of described Cypriniformes
MySearch <- rcatfish_species_by(query = "Cypriniformes") 
#Obtain the total number of genera and species available, valid, and described in the last 10 years
CofF_totals <- rcatfish_species_by(query = "Totals")
#Search for more than one family
MySearch <- rcatfish_species_by(query = c("Cichlidae","Embiotocidae"))
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R., Eschmeyer, W.N. & Fong J.D. (Year Accessed). Eschmeyer’s Catalog of Fishes: Species by family/subfamily in the Catalog of Fishes. https://researcharchive.calacademy.org/research/ichthyology/catalog/SpeciesByFamily.asp.
}
\author{
Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_updates.R
\name{rcatfish_updates}
\alias{rcatfish_updates}
\title{Access catalog changes and additions}
\usage{
rcatfish_updates(
  changes = TRUE,
  author.changes = TRUE,
  spell.changes = TRUE,
  added.genera = TRUE,
  added.species = TRUE
)
}
\arguments{
\item{changes}{Logical. Should the function return the table of changes to the catalog? Default is TRUE.}

\item{author.changes}{Logical. Should the function return the table of author and year changes to the catalog? Default is TRUE.}

\item{spell.changes}{Logical. Should the function return the table of spelling changes to the catalog? Default is TRUE.}

\item{added.genera}{Logical. Should the function return genera added to the catalog? Default is TRUE.}

\item{added.species}{Logical. Should the function return species added to the catalog? Default is TRUE.}
}
\value{
A named list. Names include UpdateDate, Changes, AuthorshipChanges, AddedGenera, and AddedSpecies. UpdateDate will always be returned and is a character string of the date the catalog was updated. Changes, AddedGenera, and AddedSpecies summarize changes to the catalog, changes to authorship information, genera added to the catalog, and species added to the catalog respectively. The elements returned are based on which parameters are set to TRUE. An itemized list describing the contents in the columns of the data returned is described below.
\itemize{
\item Original.Taxon - Character. The original taxonomic name in the database.
\item Was - Character. Previous status of the taxon in the database.
\item Previous.Taxon - Character. Previous taxonomic name in the database.
\item Now - Character. Current status of the taxon in the database following the update.
\item Current.Taxon - Character. Current taxonomic name in the database for the taxon following the update.
\item Taxon - Character. Taxon (of any rank) whose citation was updated.
\item Previous.Author - Character. Authorship information of the citation before the update.
\item Current.Author - Character. Authorship information of the citation following the update.
\item Species.Name - Character. Current taxonomic name in the database for the taxon following the update.
\item Genus.Name - Character. Name of the new genus added to the database following the update.
\item Authors - Character. Authorship for new taxonomic names added to the database following the update.
\item Year   - Character. Year for new taxonomic names added to the database following the update.
\item Region - Character. Region new taxa added to the database are known to occur in.
\item Was.Spelled - Character. Previous spelling of the taxon in the database.
\item Now.Spelled - Character. Revised spelling of the taxon in the database.
}
}
\description{
This function is used to access the Eschmeyer's Catalog of Fishes Summary of Changes and Additions.
}
\details{
This function displays the information on the "Changes and Additions" tab of Eschmeyer's Catalog of Fishes. The catalog updates this information monthly with changes to the catalog, new genus information, and new species information. Each updates removes the last month's information, so data obtained through this function is only representative of the changes and additions made since the previous update.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
\donttest{
# return all available data
myList <- rcatfish_updates(changes = TRUE, added.genera = TRUE, added.species = TRUE)
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R., van der Laan, R. & Fong, J.D. (Year Accessed). Eschmeyer’s Catalog of Fishes: Changes and Additions. https://researcharchive.calacademy.org/research/ichthyology/catalog/ChangeSummary.asp.
}
\author{
Brandon E. Dominy, Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_collections.R
\name{rcatfish_collections}
\alias{rcatfish_collections}
\title{Accesses the Eschmeyer's Catalog of Fishes Guide to Fish Collections}
\usage{
rcatfish_collections(
  query = NULL,
  abbreviation = NULL,
  country = NULL,
  phrase = FALSE,
  sleep.time = 10,
  verbose = TRUE
)
}
\arguments{
\item{query}{Character Vector of search terms to search for. Default is NULL.}

\item{abbreviation}{Character vector containing the musuem collection code(s). Default is NULL.}

\item{country}{Character vector of country names to search for. Default is NULL.}

\item{phrase}{Logical. Should query be passed as a quoted phrase (e.g. "Museum of Zoology"). Default is FALSE.}

\item{sleep.time}{Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.}

\item{verbose}{Logical. Should query progress be messaged to the screen? Default is TRUE.}
}
\value{
Data frame, in which each row is a natural history collection and each column the corresponding collection information (see itemized list below).
\itemize{
\item Code - Character. Natural history collection abbreviation code.
\item Name  - Character. Full name of the natural history collection.
\item Country  - Character. Country of the natural history collection.
\item OtherAbbr - Character. Other and previous abbreviation(s) used for that natural history collection.
\item OldName - Character. Previous name(s) used by the natural history.
\item NewName - Character. New name(s) of the natural history collection.
\item WebPage - Character. URL for the webpage of the natural history collection if one exists.
\item CollectionDB - Character. URL for accessing the collection database of the natural history collection if one exists.
\item TypesDB - Character. URL for accessing the type database of the natural history collection if one exists.
\item Remarks - Character. Additional information about the natural history collection (e.g. includes specimens from other collections, closed, etc.).
\item Publications - Character. Comma separated character vector of Eschmeyer's Catalog of Fishes reference numbers citing the natural history collection.
}
}
\description{
Accesses the Eschmeyer's Catalog of Fishes Guide to Fish Collections
}
\details{
This function retrieves and parses natural history collection data from the Eschmeyer's Catalog of
Fishes. Returned information includes the collection abbreviation, country, alternate abbreviations,
old and new collection names, websites and databases associated with the collection, remarks, and
publications associated with the collection.

It is important that search criteria match exactly how they are on the Catalog of Fishes, otherwise
data will not be found (case must match, spelling, etc.). Users wishing to query more than one query
mixed with either a country or abbreviation should note that all parameters must be of equal length, so if searching
two queries both in U.S.A., the country should be supplied twice (see the final example in the examples below).
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
#Search Museum Code For UMMZ (University of Michigan Museum of Zoology)
my.collections <- rcatfish_collections(abbreviation = "UMMZ", country = NULL, 
query = NULL, verbose = TRUE)
\donttest{
#Search For Collections in France
my.collections <- rcatfish_collections(abbreviation = NULL, country = "France", query = NULL)
#Search For Collections in The United States that are in Illinois
my.collections <- rcatfish_collections(abbreviation = NULL, country = "U.S.A.", query = "Illinois")
#Search For Collections in the United States that are in California & Alaska.
my.collections <- rcatfish_collections(abbreviation = NULL, country = rep("U.S.A.",2), 
query = c("California","Alaska"), sleep.time = 10)
#Search for Collections with Museum of Zoology in the name
my.collections <- rcatfish_collections(query = "Museum of Zoology", phrase = TRUE)
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R. & Eschmeyer, W.N. (Year Accessed). Eschmeyer's Catalog of Fishes: Guide to Fish Collections. https://researcharchive.calacademy.org/research/ichthyology/catalog/collections.asp.
}
\author{
Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_references.R
\name{rcatfish_references}
\alias{rcatfish_references}
\title{Search for references in Eschmeyer's Catalog of Fishes}
\usage{
rcatfish_references(query, type, sleep.time = 10, verbose = TRUE)
}
\arguments{
\item{query}{Either a character vector of keywords to search for or a numeric vector with reference numbers}

\item{type}{Either "RefNo" if searching for a reference number or "keyword" if searching by keywords
reference numbers.}

\item{sleep.time}{Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.}

\item{verbose}{Logical. Should query progress be messaged to the screen? Default is TRUE.}
}
\value{
A data.frame containing columns for the query, reference number, and full citation.
}
\description{
Search for references in the Catalog of Fishes by keyword or reference number.
}
\details{
This function searches Catalog of Fishes references. Users can supply either a keyword
(i.e. cichlidae, revision, etc.) or a reference number to retrieve reference information.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
\donttest{
#Perform a search of references that contain the keyword Amphilophus.
my.refs<-rcatfish_references(query = "Amphilophus", type = "keyword")
#Perform a search of references based on a Catalog of Fishes reference number
my.refs<-rcatfish_references(query = 2787, type = "RefNo")
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R. (Year Accessed). Eschmeyer's Catalog of Fishes: References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp
}
\author{
Samuel R. Borstein, Brian C. O'Meara
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_glossary.R
\name{rcatfish_glossary}
\alias{rcatfish_glossary}
\title{Access the Eschmeyer's Catalog of Fishes Glossary}
\usage{
rcatfish_glossary()
}
\value{
A dataframe with three columns. Some terms have sub-terms (e.g. synonym will have definitions for junior, objective, and subjective in addition to defining synonym itself). The first column represents the name of the main term, the second column represents the name of sub-terms, and the final column provides the definition for these terms.
}
\description{
This function is used to access the Eschmeyer's Catalog of Fishes Glossary.
}
\details{
This function displays the Glossary tab of Eschmeyer's Catalog of Fishes. The glossary contains definitions for various terms that are important to the discussion of taxonomy and are mentioned throughout the database.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
\donttest{
# Obtain the Eschmeyer's Catalog of Fishes Glossary
myGlossary <- rcatfish_glossary()
}
\dontshow{\}) # examplesIf}
}
\references{
van der Laan, R., Fricke, R. & Fong, J. (Year Accessed). Eschmeyer's Catalog of Fishes: Glossary. https://www.calacademy.org/scientists/catalog-of-fishes-glossary/.
}
\author{
Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_classification.R
\name{rcatfish_classification}
\alias{rcatfish_classification}
\title{Access the Eschmeyer's Catalog of Fishes Classification}
\usage{
rcatfish_classification()
}
\value{
A dataframe with seven columns. From left to right (and most inclusive to least inclusive), the columns represent class, order, suborder, family, subfamily, author, and the common name of the family or subfamily.
}
\description{
This function is used to access the Eschmeyer's Catalog of Fishes classification of fishes
}
\details{
This function displays the classification tab of Eschmeyer's Catalog of Fishes. The data are returned in a tabular framework that progresses from most to least inclusive from left to right. The function will return the class, order, suborder, family, subfamily, author, and common name of the family or subfamily.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
#Obtain the Eschmeyer's Catalog of Fishes Classification
\donttest{
myClassification <- rcatfish_classification()
}
\dontshow{\}) # examplesIf}
}
\references{
van der Laan, R., Fricke, R. & Eschmeyer, W.N. (Year Accessed). Eschmeyer's Catalog of Fishes: Classification. https://www.calacademy.org/scientists/catalog-of-fishes-classification/.
}
\author{
Samuel R. Borstein
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcatfish_search.R
\name{rcatfish_search}
\alias{rcatfish_search}
\title{Search for genera and species in the Eschmeyer's Catalog of Fishes}
\usage{
rcatfish_search(
  query,
  type,
  unavailable = FALSE,
  taxon.history = FALSE,
  resolve = FALSE,
  sleep.time = 10,
  phrase = FALSE,
  verbose = TRUE,
  common.name = FALSE,
  language = "English"
)
}
\arguments{
\item{query}{Character or Character Vector containing the name or names of the taxon to search. Note, you can not mix common and scientific names as a query.}

\item{type}{Character either "Genus" or "Species" to search for genera and species respectively.
Note that only one of these options can be chosen.}

\item{unavailable}{Logical. Should the query be run with unavailable names include? Default is FALSE.}

\item{taxon.history}{Should a detailed history of taxonomic changes per taxa be returned (i.e. synonymization, raised to validity, authority, etc.). Default is FALSE.}

\item{resolve}{Logical. If a match for the query isn't found, should an attempt be made to resolve the name using taxize?}

\item{sleep.time}{Numeric. Time in seconds to sleep between query calls to the California Academy of Sciences page. This is set by default to 10 seconds, which is in their robots.txt. Adjust at your own risk.}

\item{phrase}{Logical. Should query be passed as a quoted phrase (e.g. "Synonym of Cyprinus carpio"). Default is FALSE.}

\item{verbose}{Logical. Should query progress be messaged to the screen? Default is TRUE.}

\item{common.name}{Is the query a common name? Common names will be converted to scientific names for searching through rfishbase. Note, you can not mix common and scientific names as a query. This likely will only work for species searches. Default is FALSE.}

\item{language}{Language to perform common name search. Default is English.}
}
\value{
Data frames. If taxon.history = TRUE, a list of two data frames. In this case, the first data frame TaxonSummary contains information on the description and current
status of the taxa in the query, references to descriptions, and information on the type locality, types, family/subfamily, distribution, and habitat for species and type species
gender, status, and authorities for genera searches. The second data frame, TaxonHistory contains detailed information on the taxonomic history
of a taxon, such as which authorities have viewed it as a synonym or valid since its description. An itemized list describing the contents in the columns of the data returned is described below.
\itemize{
\item Query - Character. The submitted query.
\item Nominal Taxa - Character. Nominal taxonomic names.
\item Author - Character. Authorship of the species/Genus description.
\item DescriptionRef - Character. Eschmeyer Catalog of Fishes reference number for the genus or species description.
\item DescriptionYear - Numeric. Year in which taxon was described.
\item Status - Character. Current status of the nominal taxon.
\item CurrentNomenclature - Character. Currently recognized taxonomic name of the taxon.
\item CurrentAuthority - Character. Current authority for the valid name of the taxon.
\item Holotype - Character. Catalog number of the holotype.
\item Paratype - Character. Catalog number(s) of the paratypes.
\item Lectotype - Character. Catalog number of the lectotype.
\item Paralectotype - Character. Catalog number(s) of the paralectotype.
\item Neotype - Character. Catalog number of the neotype.
\item Syntype - Character. Catalog number(s) of the syntypes.
\item NoTypes - Character. Specifies entries with currently no known types.
\item TypeLocality - Character. Type locality of the taxon.
\item Family - Character. Family the taxon belongs to.
\item Subfamily - Character. Subfamily the taxon belongs to.
\item Distribution - Character. Distribution of the species.
\item Fresh - Numeric. Binary presence (1) or absence (0) in freshwater.
\item Brackish - Numeric. Binary presence (1) or absence (0) in brackish water.
\item Marine - Numeric. Binary presence (1) or absence (0) in marine water.
\item IUCNYear - Numeric. Year in which IUCN status was assessed.
\item IUCNStatus - Character. Status in the IUCN list of threatened species.
\item NomenclatureNotes - Character. Descriptive tags identifying status as a homonym, hybrid, nomen protectum, etc.
\item TypeSpecies - Character. Type Species of the genus.
\item Gender - Character. Gender of the taxon.
\item TypeBy - Character. Type designation.
\item Notes - Character. Any notes related to the taxon or taxonomic history (e.g. treated as a subspecies, availability of name, authorship issues, etc.).
\item AsSubgenus - Character. If the taxon was described as a subgenus of another genus, provides information on the genus.
\item Infrasubspecific - Character. Infrasubspecific designation if it exists.
\item Miscellaneous - Character. Additional information not captured in other columns.
\item ResultCode - Character. A unique identifier for each nominal taxa, useful for linking table rows when homonyms are present.
}
}
\description{
Search for genera and species in the Eschmeyer's Catalog of Fishes
}
\details{
This function searches for genera or species in the Catalog of Fishes and returns its
valid status, synonyms, and taxonomic history as well as reference numbers for the authority of
the citations. By default, the function returns basic information on a taxon, such as who described it, its current taxonomic status, type locality,
gender of the name, etc. If users choose taxon.history = TRUE, a detailed list of taxonomic information regarding nomenclature acts associated with the taxa is also returned.
Note that the function will take longer to run, sometimes twice as long if taxon.history = TRUE.

One problem a user may encounter using the Catalog of Fishes website is that the input taxon name must match directly to a term in the database
or the database will not return any information. While this remains true using this package, users can attempt to resolve names by setting resolve = TRUE.
When resolve = TRUE, rcatfish_search will use the Global Names Verifier (GNverifier) in an attempt to resolve the name, which will then be passed to downstream function calls.
This is meant to be useful, but we recommend using this option be cautious about what the GNverifier returns. A message will print to the screen notifying you what
name the GNverifier resolved to be the best match and will be used, though we strongly recommend users check the resolved name does not deviate from their expectations (i.e. a homonym or similar name for a different group is not returned).
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
#Note that for Windows OS, OpenSSL must be used as a backend for curl. 
#Please see vignette on how to do this with vignette('rcatfish').

#Search for Abactochromis and return taxon history

if((.Platform$OS.type == "windows") & (grepl(pattern = "\\\\(OpenSSL", 
  curl::curl_version()$ssl_version) == TRUE)){
cat("openSSL backend for curl is required for the Windows version of this package, but it not 
detected as being active. Please see the vignette on how to configure curl with openSSL for this
function to work. You can access the vignette with the following: vignette('rcatfish').")
}else{
my.search <- rcatfish_search(type = "Species", query = "Abactochromis", 
  taxon.history = FALSE, resolve = FALSE, sleep.time = 0)
}

#Search for the genera Astatheros and Gasteropelecidae
\donttest{
if((.Platform$OS.type == "windows") & (grepl(pattern = "\\\\(OpenSSL", 
  curl::curl_version()$ssl_version) == TRUE)){
cat("openSSL backend for curl is required for the Windows version of this package, but it not 
detected as being active. Please see the vignette on how to configure curl with openSSL for this
function to work. You can access the vignette with the following: vignette('rcatfish').")
}else{
  my.search <- rcatfish_search(type = "Genus", query = c("Astatheros","Gasteropelecidae"), 
    taxon.history = TRUE, resolve = FALSE, sleep.time = 10)
  }

#Perform a species search for two different taxa

if((.Platform$OS.type == "windows") & (grepl(pattern = "\\\\(OpenSSL", 
  curl::curl_version()$ssl_version) == TRUE)){
cat("openSSL backend for curl is required for the Windows version of this package, but it not 
detected as being active. Please see the vignette on how to configure curl with openSSL for this
function to work. You can access the vignette with the following: vignette('rcatfish').")
}else{
my.search<-rcatfish_search(type = "Species", query = c("Ctenopharynx",
  "Pseudocrenilabrus multicolor victoriae"), taxon.history = TRUE, resolve = FALSE, 
  sleep.time = 10)
  }
}
\dontshow{\}) # examplesIf}
}
\references{
Fricke, R., Eschmeyer, W.N. & van der Laan, R. (Year Accessed). Eschmeyer's Catalog of Fishes: Genera, Species, References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp

Mozzherin, D. (2025). GNverifier -- a reconciler and resolver of scientific names against more than 100 data sources. (v1.3.0). Zenodo. https://doi.org/10.5281/zenodo.17245658
}
\author{
Samuel R. Borstein, Brandon E. Dominy, Brian C. O'Meara
}
